@@ -36,12 +36,16 @@ const (
 	CMEventTrackingEnabled    = PrefixEvent + "trackingEnabled"    // Application Tracking
 	CMEventRequestCapacity    = PrefixEvent + "requestCapacity"    // Request Capacity
 	CMEventRingBufferCapacity = PrefixEvent + "ringBufferCapacity" // Ring Buffer Capacity
+	CMMaxEventStreams         = PrefixEvent + "maxStreams"
+	CMMaxEventStreamsPerHost  = PrefixEvent + "maxStreamsPerHost"
 
 	// defaults
 	DefaultHealthCheckInterval     = 30 * time.Second
 	DefaultEventTrackingEnabled    = true
 	DefaultEventRequestCapacity    = 1000
 	DefaultEventRingBufferCapacity = 100000
+	DefaultMaxStreams              = uint64(100)
+	DefaultMaxStreamsPerHost       = uint64(15)
 )
 
 var ConfigContext *SchedulerConfigContext
