@@ -22,12 +22,12 @@ import (
 	"sync"
 
 	"github.com/apache/yunikorn-core/pkg/entrypoint"
-	"github.com/apache/yunikorn-core/pkg/scheduler/tests"
+	"github.com/apache/yunikorn-core/pkg/mock"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
 type exampleRMCallback struct {
-	tests.MockResourceManagerCallback
+	mock.ResourceManagerCallback
 	acceptedApplications map[string]bool
 	rejectedApplications map[string]bool
 	acceptedNodes        map[string]bool
@@ -42,7 +42,7 @@ func (m *exampleRMCallback) UpdateAllocation(response *si.AllocationResponse) er
 	m.Lock()
 	defer m.Unlock()
 	for _, alloc := range response.New {
-		m.Allocations[alloc.UUID] = alloc
+		m.Allocations[alloc.AllocationID] = alloc
 		if val, ok := m.nodeAllocations[alloc.NodeID]; ok {
 			val = append(val, alloc)
 			m.nodeAllocations[alloc.NodeID] = val
@@ -54,7 +54,7 @@ func (m *exampleRMCallback) UpdateAllocation(response *si.AllocationResponse) er
 	}
 
 	for _, alloc := range response.Released {
-		delete(m.Allocations, alloc.UUID)
+		delete(m.Allocations, alloc.AllocationID)
 	}
 	return nil
 }
@@ -205,7 +205,7 @@ partitions:
 		panic(err)
 	}
 
-	// Refer to mock_rm_callback.go:109
+	// Refer to mock/rm_callback.go:109
 	// You need to check app accepted by scheduler before proceed.
 
 	// Send request
