@@ -28,124 +28,141 @@ import (
 	dto "github.com/prometheus/client_model/go"
 )
 
-var cqm CoreQueueMetrics
+var qm *QueueMetrics
 
 func TestApplicationsRunning(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.IncQueueApplicationsRunning()
+	qm.IncQueueApplicationsRunning()
 	verifyAppMetrics(t, "running")
 }
 
 func TestApplicationsAccepted(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.IncQueueApplicationsAccepted()
+	qm.IncQueueApplicationsAccepted()
 	verifyAppMetrics(t, "accepted")
 }
 
 func TestApplicationsRejected(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.IncQueueApplicationsRejected()
+	qm.IncQueueApplicationsRejected()
 	verifyAppMetrics(t, "rejected")
 }
 
 func TestApplicationsFailed(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.IncQueueApplicationsFailed()
+	qm.IncQueueApplicationsFailed()
 	verifyAppMetrics(t, "failed")
 }
 
 func TestApplicationsCompleted(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.IncQueueApplicationsCompleted()
+	qm.IncQueueApplicationsCompleted()
 	verifyAppMetrics(t, "completed")
 }
 
 func TestAllocatedContainers(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.IncAllocatedContainer()
+	qm.IncAllocatedContainer()
 	verifyContainerMetrics(t, "allocated", float64(1))
 }
 
 func TestReleasedContainers(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.IncReleasedContainer()
+	qm.IncReleasedContainer()
 	verifyContainerMetrics(t, "released", float64(1))
 }
 
 func TestAddReleasedContainers(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.AddReleasedContainers(2)
+	qm.AddReleasedContainers(2)
 	verifyContainerMetrics(t, "released", float64(2))
 }
 
 func TestQueueGuaranteedResourceMetrics(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.SetQueueGuaranteedResourceMetrics("cpu", 1)
+	qm.SetQueueGuaranteedResourceMetrics("cpu", 1)
 	verifyResourceMetrics(t, "guaranteed", "cpu")
 }
 
 func TestQueueMaxResourceMetrics(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.SetQueueMaxResourceMetrics("cpu", 1)
+	qm.SetQueueMaxResourceMetrics("cpu", 1)
 	verifyResourceMetrics(t, "max", "cpu")
 }
 
 func TestQueueAllocatedResourceMetrics(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.SetQueueAllocatedResourceMetrics("cpu", 1)
+	qm.SetQueueAllocatedResourceMetrics("cpu", 1)
 	verifyResourceMetrics(t, "allocated", "cpu")
 }
 
 func TestQueuePendingResourceMetrics(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterQueueMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.SetQueuePendingResourceMetrics("cpu", 1)
+	qm.SetQueuePendingResourceMetrics("cpu", 1)
 	verifyResourceMetrics(t, "pending", "cpu")
 }
 
 func TestQueuePreemptingResourceMetrics(t *testing.T) {
-	cqm = getQueueMetrics()
-	defer unregisterMetrics(t)
+	qm = getQueueMetrics()
+	defer unregisterQueueMetrics()
 
-	cqm.SetQueuePreemptingResourceMetrics("cpu", 1)
+	qm.SetQueuePreemptingResourceMetrics("cpu", 1)
 	verifyResourceMetrics(t, "preempting", "cpu")
 }
 
-func getQueueMetrics() CoreQueueMetrics {
+func getQueueMetrics() *QueueMetrics {
 	return InitQueueMetrics("root.test")
 }
 
 func verifyAppMetrics(t *testing.T, expectedState string) {
+	verifyAppMetricsLabel(t, expectedState)
+	verifyAppMetricsSubsystem(t, expectedState)
+}
+
+func verifyAppMetricsLabel(t *testing.T, expectedState string) {
+	checkFn := func(labels []*dto.LabelPair) {
+		assert.Equal(t, 2, len(labels))
+		assert.Equal(t, "queue", *labels[0].Name)
+		assert.Equal(t, "root.test", *labels[0].Value)
+		assert.Equal(t, "state", *labels[1].Name)
+		assert.Equal(t, expectedState, *labels[1].Value)
+	}
+
+	verifyMetricsLabel(t, checkFn)
+}
+
+func verifyAppMetricsSubsystem(t *testing.T, expectedState string) {
 	checkFn := func(labels []*dto.LabelPair) {
 		assert.Equal(t, 1, len(labels))
 		assert.Equal(t, "state", *labels[0].Name)
 		assert.Equal(t, expectedState, *labels[0].Value)
 	}
 
-	verifyMetrics(t, checkFn)
+	verifyMetricsSubsytem(t, checkFn)
 }
 
 func verifyContainerMetrics(t *testing.T, expectedState string, value float64) {
@@ -171,6 +188,25 @@ func verifyContainerMetrics(t *testing.T, expectedState string, value float64) {
 	assert.Assert(t, checked, "Failed to find metric")
 }
 func verifyResourceMetrics(t *testing.T, expectedState, expectedResource string) {
+	verifyResourceMetricsLabel(t, expectedState, expectedResource)
+	verifyResourceMetricsSubsystem(t, expectedState, expectedResource)
+}
+
+func verifyResourceMetricsLabel(t *testing.T, expectedState, expectedResource string) {
+	checkFn := func(labels []*dto.LabelPair) {
+		assert.Equal(t, 3, len(labels))
+		assert.Equal(t, "queue", *labels[0].Name)
+		assert.Equal(t, "root.test", *labels[0].Value)
+		assert.Equal(t, "resource", *labels[1].Name)
+		assert.Equal(t, expectedResource, *labels[1].Value)
+		assert.Equal(t, "state", *labels[2].Name)
+		assert.Equal(t, expectedState, *labels[2].Value)
+	}
+
+	verifyMetricsLabel(t, checkFn)
+}
+
+func verifyResourceMetricsSubsystem(t *testing.T, expectedState, expectedResource string) {
 	checkFn := func(labels []*dto.LabelPair) {
 		assert.Equal(t, 2, len(labels))
 		assert.Equal(t, "resource", *labels[0].Name)
@@ -179,10 +215,31 @@ func verifyResourceMetrics(t *testing.T, expectedState, expectedResource string)
 		assert.Equal(t, expectedState, *labels[1].Value)
 	}
 
-	verifyMetrics(t, checkFn)
+	verifyMetricsSubsytem(t, checkFn)
 }
 
-func verifyMetrics(t *testing.T, checkLabel func(label []*dto.LabelPair)) {
+func verifyMetricsLabel(t *testing.T, checkLabel func(label []*dto.LabelPair)) {
+	mfs, err := prometheus.DefaultGatherer.Gather()
+	assert.NilError(t, err)
+
+	var checked bool
+	for _, metric := range mfs {
+		if strings.Contains(metric.GetName(), "yunikorn_queue") {
+			assert.Equal(t, 1, len(metric.Metric))
+			assert.Equal(t, dto.MetricType_GAUGE, metric.GetType())
+			m := metric.Metric[0]
+			checkLabel(m.Label)
+			assert.Assert(t, m.Gauge != nil)
+			assert.Equal(t, float64(1), *m.Gauge.Value)
+			checked = true
+			break
+		}
+	}
+
+	assert.Assert(t, checked, "Failed to find metric")
+}
+
+func verifyMetricsSubsytem(t *testing.T, checkLabel func(label []*dto.LabelPair)) {
 	mfs, err := prometheus.DefaultGatherer.Gather()
 	assert.NilError(t, err)
 
@@ -203,13 +260,10 @@ func verifyMetrics(t *testing.T, checkLabel func(label []*dto.LabelPair)) {
 	assert.Assert(t, checked, "Failed to find metric")
 }
 
-func unregisterQueueMetrics(t *testing.T) {
-	qm, ok := cqm.(*QueueMetrics)
-	if !ok {
-		t.Fatalf("Type assertion failed, metrics is not QueueMetrics")
-	}
-
-	prometheus.Unregister(qm.appMetrics)
+func unregisterQueueMetrics() {
+	prometheus.Unregister(qm.appMetricsLabel)
+	prometheus.Unregister(qm.appMetricsSubsystem)
 	prometheus.Unregister(qm.containerMetrics)
-	prometheus.Unregister(qm.ResourceMetrics)
+	prometheus.Unregister(qm.resourceMetricsLabel)
+	prometheus.Unregister(qm.resourceMetricsSubsystem)
 }
