@@ -40,6 +40,7 @@ import (
 // Gateway to talk to ResourceManager (behind grpc/API of scheduler-interface)
 type RMProxy struct {
 	EventHandlers handler.EventHandlers
+	stop          chan struct{}
 
 	// Internal fields
 	pendingRMEvents chan interface{}
@@ -74,6 +75,7 @@ func NewRMProxy() *RMProxy {
 	rm := &RMProxy{
 		rmIDToCallback:  make(map[string]api.ResourceManagerCallback),
 		pendingRMEvents: make(chan interface{}, 1024*1024),
+		stop:            make(chan struct{}),
 	}
 	return rm
 }
@@ -228,24 +230,28 @@ func (rmp *RMProxy) processRMNodeUpdateEvent(event *rmevent.RMNodeUpdateEvent) {
 
 func (rmp *RMProxy) handleRMEvents() {
 	for {
-		ev := <-rmp.pendingRMEvents
-		switch v := ev.(type) {
-		case *rmevent.RMNewAllocationsEvent:
-			rmp.processAllocationUpdateEvent(v)
-		case *rmevent.RMApplicationUpdateEvent:
-			rmp.processApplicationUpdateEvent(v)
-		case *rmevent.RMReleaseAllocationEvent:
-			rmp.processRMReleaseAllocationEvent(v)
-		case *rmevent.RMRejectedAllocationAskEvent:
-			rmp.processRMRejectedAllocationAskEvent(v)
-		case *rmevent.RMRejectedAllocationEvent:
-			rmp.processRMRejectedAllocationEvent(v)
-		case *rmevent.RMNodeUpdateEvent:
-			rmp.processRMNodeUpdateEvent(v)
-		case *rmevent.RMReleaseAllocationAskEvent:
-			rmp.processRMReleaseAllocationAskEvent(v)
-		default:
-			panic(fmt.Sprintf("%s is not an acceptable type for RM event.", reflect.TypeOf(v).String()))
+		select {
+		case ev := <-rmp.pendingRMEvents:
+			switch v := ev.(type) {
+			case *rmevent.RMNewAllocationsEvent:
+				rmp.processAllocationUpdateEvent(v)
+			case *rmevent.RMApplicationUpdateEvent:
+				rmp.processApplicationUpdateEvent(v)
+			case *rmevent.RMReleaseAllocationEvent:
+				rmp.processRMReleaseAllocationEvent(v)
+			case *rmevent.RMRejectedAllocationAskEvent:
+				rmp.processRMRejectedAllocationAskEvent(v)
+			case *rmevent.RMRejectedAllocationEvent:
+				rmp.processRMRejectedAllocationEvent(v)
+			case *rmevent.RMNodeUpdateEvent:
+				rmp.processRMNodeUpdateEvent(v)
+			case *rmevent.RMReleaseAllocationAskEvent:
+				rmp.processRMReleaseAllocationAskEvent(v)
+			default:
+				panic(fmt.Sprintf("%s is not an acceptable type for RM event.", reflect.TypeOf(v).String()))
+			}
+		case <-rmp.stop:
+			return
 		}
 	}
 }
@@ -308,37 +314,26 @@ func (rmp *RMProxy) UpdateAllocation(request *si.AllocationRequest) error {
 	if rmp.GetResourceManagerCallback(request.RmID) == nil {
 		return fmt.Errorf("received AllocationRequest, but RmID=\"%s\" not registered", request.RmID)
 	}
-	go func() {
-		// Update allocations
-		if len(request.Allocations) > 0 {
-			for _, alloc := range request.Allocations {
-				alloc.PartitionName = common.GetNormalizedPartitionName(alloc.PartitionName, request.RmID)
-			}
-		}
+	// Update allocations
+	for _, alloc := range request.Allocations {
+		alloc.PartitionName = common.GetNormalizedPartitionName(alloc.PartitionName, request.RmID)
+	}
 
-		// Update asks
-		if len(request.Asks) > 0 {
-			for _, ask := range request.Asks {
-				ask.PartitionName = common.GetNormalizedPartitionName(ask.PartitionName, request.RmID)
-			}
-		}
+	// Update asks
+	for _, ask := range request.Asks {
+		ask.PartitionName = common.GetNormalizedPartitionName(ask.PartitionName, request.RmID)
+	}
 
-		// Update releases
-		if request.Releases != nil {
-			if len(request.Releases.AllocationsToRelease) > 0 {
-				for _, rel := range request.Releases.AllocationsToRelease {
-					rel.PartitionName = common.GetNormalizedPartitionName(rel.PartitionName, request.RmID)
-				}
-			}
-
-			if len(request.Releases.AllocationAsksToRelease) > 0 {
-				for _, rel := range request.Releases.AllocationAsksToRelease {
-					rel.PartitionName = common.GetNormalizedPartitionName(rel.PartitionName, request.RmID)
-				}
-			}
+	// Update releases
+	if request.Releases != nil {
+		for _, rel := range request.Releases.AllocationsToRelease {
+			rel.PartitionName = common.GetNormalizedPartitionName(rel.PartitionName, request.RmID)
 		}
-		rmp.EventHandlers.SchedulerEventHandler.HandleEvent(&rmevent.RMUpdateAllocationEvent{Request: request})
-	}()
+		for _, rel := range request.Releases.AllocationAsksToRelease {
+			rel.PartitionName = common.GetNormalizedPartitionName(rel.PartitionName, request.RmID)
+		}
+	}
+	rmp.EventHandlers.SchedulerEventHandler.HandleEvent(&rmevent.RMUpdateAllocationEvent{Request: request})
 	return nil
 }
 
@@ -347,21 +342,17 @@ func (rmp *RMProxy) UpdateApplication(request *si.ApplicationRequest) error {
 		return fmt.Errorf("received ApplicationRequest, but RmID=\"%s\" not registered", request.RmID)
 	}
 
-	go func() {
-		// Update New apps
-		if len(request.New) > 0 {
-			for _, app := range request.New {
-				app.PartitionName = common.GetNormalizedPartitionName(app.PartitionName, request.RmID)
-			}
-		}
-		// Update Remove apps
-		if len(request.Remove) > 0 {
-			for _, app := range request.Remove {
-				app.PartitionName = common.GetNormalizedPartitionName(app.PartitionName, request.RmID)
-			}
-		}
-		rmp.EventHandlers.SchedulerEventHandler.HandleEvent(&rmevent.RMUpdateApplicationEvent{Request: request})
-	}()
+	// Update New apps
+	for _, app := range request.New {
+		app.PartitionName = common.GetNormalizedPartitionName(app.PartitionName, request.RmID)
+	}
+
+	// Update Remove apps
+	for _, app := range request.Remove {
+		app.PartitionName = common.GetNormalizedPartitionName(app.PartitionName, request.RmID)
+	}
+
+	rmp.EventHandlers.SchedulerEventHandler.HandleEvent(&rmevent.RMUpdateApplicationEvent{Request: request})
 	return nil
 }
 
@@ -369,18 +360,16 @@ func (rmp *RMProxy) UpdateNode(request *si.NodeRequest) error {
 	if rmp.GetResourceManagerCallback(request.RmID) == nil {
 		return fmt.Errorf("received NodeRequest, but RmID=\"%s\" not registered", request.RmID)
 	}
-	go func() {
-		if len(request.Nodes) > 0 {
-			for _, node := range request.Nodes {
-				if len(node.GetAttributes()) == 0 {
-					node.Attributes = map[string]string{}
-				}
-				partition := node.Attributes[siCommon.NodePartition]
-				node.Attributes[siCommon.NodePartition] = common.GetNormalizedPartitionName(partition, request.RmID)
-			}
+
+	for _, node := range request.Nodes {
+		if len(node.GetAttributes()) == 0 {
+			node.Attributes = map[string]string{}
 		}
-		rmp.EventHandlers.SchedulerEventHandler.HandleEvent(&rmevent.RMUpdateNodeEvent{Request: request})
-	}()
+		partition := node.Attributes[siCommon.NodePartition]
+		node.Attributes[siCommon.NodePartition] = common.GetNormalizedPartitionName(partition, request.RmID)
+	}
+
+	rmp.EventHandlers.SchedulerEventHandler.HandleEvent(&rmevent.RMUpdateNodeEvent{Request: request})
 	return nil
 }
 
@@ -407,3 +396,8 @@ func (rmp *RMProxy) UpdateConfiguration(request *si.UpdateConfigurationRequest)
 	}
 	return nil
 }
+
+func (rmp *RMProxy) Stop() {
+	log.Log(log.RMProxy).Info("Stopping RMProxy")
+	close(rmp.stop)
+}
