@@ -144,7 +144,7 @@ func (cc *ClusterContext) schedule() bool {
 			metrics.GetSchedulerMetrics().ObserveSchedulingLatency(schedulingStart)
 			if alloc.GetResult() == objects.Replaced {
 				// communicate the removal to the RM
-				cc.notifyRMAllocationReleased(psc.RmID, alloc.GetReleasesClone(), si.TerminationType_PLACEHOLDER_REPLACED, "replacing uuid: "+alloc.GetUUID())
+				cc.notifyRMAllocationReleased(psc.RmID, alloc.GetReleasesClone(), si.TerminationType_PLACEHOLDER_REPLACED, "replacing allocationID: "+alloc.GetAllocationID())
 			} else {
 				cc.notifyRMNewAllocation(psc.RmID, alloc)
 			}
@@ -600,7 +600,7 @@ func (cc *ClusterContext) addNode(nodeInfo *si.NodeInfo, schedulable bool) error
 	partition := cc.GetPartition(sn.Partition)
 	if partition == nil {
 		err := fmt.Errorf("failed to find partition %s for new node %s", sn.Partition, sn.NodeID)
-		//nolint: TODO assess impact of partition metrics (this never hit the partition)
+		//nolint:godox //TODO: assess impact of partition metrics (this never hit the partition)
 		metrics.GetSchedulerMetrics().IncFailedNodes()
 		log.Log(log.SchedContext).Error("Failed to add node to non existing partition",
 			zap.String("nodeID", sn.NodeID),
@@ -788,7 +788,9 @@ func (cc *ClusterContext) processAllocations(request *si.AllocationRequest) {
 				zap.String("applicationID", siAlloc.ApplicationID),
 				zap.String("allocationKey", siAlloc.AllocationKey),
 				zap.Error(err))
+			continue
 		}
+		cc.notifyRMNewAllocation(request.RmID, alloc)
 	}
 
 	// Reject allocs returned to RM proxy for the apps and partitions not found
@@ -921,7 +923,7 @@ func (cc *ClusterContext) notifyRMAllocationReleased(rmID string, released []*ob
 		releaseEvent.ReleasedAllocations = append(releaseEvent.ReleasedAllocations, &si.AllocationRelease{
 			ApplicationID:   alloc.GetApplicationID(),
 			PartitionName:   alloc.GetPartitionName(),
-			UUID:            alloc.GetUUID(),
+			AllocationID:    alloc.GetAllocationID(),
 			TerminationType: terminationType,
 			Message:         message,
 			AllocationKey:   alloc.GetAllocationKey(),
@@ -984,3 +986,11 @@ func (cc *ClusterContext) SetLastHealthCheckResult(c *dao.SchedulerHealthDAOInfo
 func (cc *ClusterContext) GetUUID() string {
 	return cc.uuid
 }
+
+func (cc *ClusterContext) Stop() {
+	log.Log(log.SchedContext).Info("Stopping background services of partitions")
+	for _, part := range cc.GetPartitionMapClone() {
+		part.partitionManager.Stop()
+		part.userGroupCache.Stop()
+	}
+}
