@@ -26,6 +26,7 @@ import (
 	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
+	"github.com/apache/yunikorn-core/pkg/events/mock"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/common"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
@@ -57,6 +58,7 @@ func TestNewAsk(t *testing.T) {
 	askStr := ask.String()
 	expected := "allocationKey ask-1, applicationID app-1, Resource map[first:10], PendingRepeats 1"
 	assert.Equal(t, askStr, expected, "Strings should have been equal")
+	assert.Equal(t, "app-1|ask-1", ask.resKeyWithoutNode) //nolint:staticcheck
 }
 
 func TestPendingAskRepeat(t *testing.T) {
@@ -228,6 +230,28 @@ func TestAllocationLog(t *testing.T) {
 	assert.Equal(t, 1, int(log[1].Count), "wrong count for event 2")
 }
 
+func TestSendPredicateFailed(t *testing.T) {
+	res := resources.NewResourceFromMap(map[string]resources.Quantity{"first": 10})
+	siAsk := &si.AllocationAsk{
+		AllocationKey:  "ask-1",
+		ApplicationID:  "app-1",
+		MaxAllocations: 1,
+		ResourceAsk:    res.ToProto(),
+	}
+	ask := NewAllocationAskFromSI(siAsk)
+	eventSystem := mock.NewEventSystemDisabled()
+	ask.askEvents = newAskEvents(ask, eventSystem)
+	ask.SendPredicateFailedEvent("failed")
+	assert.Equal(t, 0, len(eventSystem.Events))
+
+	eventSystem = mock.NewEventSystem()
+	ask.askEvents = newAskEvents(ask, eventSystem)
+	ask.SendPredicateFailedEvent("failure")
+	assert.Equal(t, 1, len(eventSystem.Events))
+	event := eventSystem.Events[0]
+	assert.Equal(t, "Predicate failed for request 'ask-1' with message: 'failure'", event.Message)
+}
+
 func sortedLog(ask *AllocationAsk) []*AllocationLogEntry {
 	log := ask.GetAllocationLog()
 	sort.SliceStable(log, func(i int, j int) bool {
