@@ -51,11 +51,15 @@ var (
 	defaultPlaceholderTimeout = 15 * time.Minute
 )
 
-var rateLimitedLog = log.RateLimitedLog(log.SchedApplication, time.Second)
+var initAppLogOnce sync.Once
+var rateLimitedAppLog *log.RateLimitedLogger
 
 const (
 	Soft string = "Soft"
 	Hard string = "Hard"
+
+	NotEnoughUserQuota  = "Not enough user quota"
+	NotEnoughQueueQuota = "Not enough queue quota"
 )
 
 type PlaceholderData struct {
@@ -87,8 +91,9 @@ type Application struct {
 	user              security.UserGroup        // owner of the application
 	allocatedResource *resources.Resource       // total allocated resources
 
-	usedResource      *resources.TrackedResource // keep track of resource usage of the application
-	preemptedResource *resources.TrackedResource // keep track of preempted resource usage of the application
+	usedResource        *resources.TrackedResource // keep track of resource usage of the application
+	preemptedResource   *resources.TrackedResource // keep track of preempted resource usage of the application
+	placeholderResource *resources.TrackedResource // keep track of placeholder resource usage of the application
 
 	maxAllocatedResource *resources.Resource         // max allocated resources
 	allocatedPlaceholder *resources.Resource         // total allocated placeholder resources
@@ -107,6 +112,8 @@ type Application struct {
 	placeholderData      map[string]*PlaceholderData // track placeholder and gang related info
 	askMaxPriority       int32                       // highest priority value of outstanding asks
 	hasPlaceholderAlloc  bool                        // Whether there is at least one allocated placeholder
+	runnableInQueue      bool                        // whether the application is runnable/schedulable in the queue. Default is true.
+	runnableByUserLimit  bool                        // whether the application is runnable/schedulable based on user/group quota. Default is true.
 
 	rmEventHandler        handler.EventHandler
 	rmID                  string
@@ -117,51 +124,25 @@ type Application struct {
 	sync.RWMutex
 }
 
-type ApplicationSummary struct {
-	ApplicationID     string
-	SubmissionTime    time.Time
-	StartTime         time.Time
-	FinishTime        time.Time
-	User              string
-	Queue             string
-	State             string
-	RmID              string
-	ResourceUsage     *resources.TrackedResource
-	PreemptedResource *resources.TrackedResource
-}
-
-func (as *ApplicationSummary) DoLogging() {
-	log.Log(log.SchedAppUsage).Info("YK_APP_SUMMARY:",
-		zap.String("appID", as.ApplicationID),
-		zap.Int64("submissionTime", as.SubmissionTime.UnixMilli()),
-		zap.Int64("startTime", as.StartTime.UnixMilli()),
-		zap.Int64("finishTime", as.FinishTime.UnixMilli()),
-		zap.String("user", as.User),
-		zap.String("queue", as.Queue),
-		zap.String("state", as.State),
-		zap.String("rmID", as.RmID),
-		zap.Any("resourceUsage", as.ResourceUsage.TrackedResourceMap),
-		zap.Any("preemptedResource", as.PreemptedResource.TrackedResourceMap),
-	)
-}
-
 func (sa *Application) GetApplicationSummary(rmID string) *ApplicationSummary {
 	sa.RLock()
 	defer sa.RUnlock()
 	state := sa.stateMachine.Current()
-	ru := sa.usedResource.Clone()
-	pu := sa.preemptedResource.Clone()
+	resourceUsage := sa.usedResource.Clone()
+	preemptedUsage := sa.preemptedResource.Clone()
+	placeHolderUsage := sa.placeholderResource.Clone()
 	appSummary := &ApplicationSummary{
-		ApplicationID:     sa.ApplicationID,
-		SubmissionTime:    sa.SubmissionTime,
-		StartTime:         sa.startTime,
-		FinishTime:        sa.finishedTime,
-		User:              sa.user.User,
-		Queue:             sa.queuePath,
-		State:             state,
-		RmID:              rmID,
-		ResourceUsage:     ru,
-		PreemptedResource: pu,
+		ApplicationID:       sa.ApplicationID,
+		SubmissionTime:      sa.SubmissionTime,
+		StartTime:           sa.startTime,
+		FinishTime:          sa.finishedTime,
+		User:                sa.user.User,
+		Queue:               sa.queuePath,
+		State:               state,
+		RmID:                rmID,
+		ResourceUsage:       resourceUsage,
+		PreemptedResource:   preemptedUsage,
+		PlaceholderResource: placeHolderUsage,
 	}
 	return appSummary
 }
@@ -177,6 +158,7 @@ func NewApplication(siApp *si.AddApplicationRequest, ugi security.UserGroup, eve
 		allocatedResource:     resources.NewResource(),
 		usedResource:          resources.NewTrackedResource(),
 		preemptedResource:     resources.NewTrackedResource(),
+		placeholderResource:   resources.NewTrackedResource(),
 		maxAllocatedResource:  resources.NewResource(),
 		allocatedPlaceholder:  resources.NewResource(),
 		requests:              make(map[string]*AllocationAsk),
@@ -191,6 +173,8 @@ func NewApplication(siApp *si.AddApplicationRequest, ugi security.UserGroup, eve
 		askMaxPriority:        configs.MinPriority,
 		sortedRequests:        sortedRequests{},
 		sendStateChangeEvents: true,
+		runnableByUserLimit:   true,
+		runnableInQueue:       true,
 	}
 	placeholderTimeout := common.ConvertSITimeoutWithAdjustment(siApp, defaultPlaceholderTimeout)
 	gangSchedStyle := siApp.GetGangSchedulingStyle()
@@ -691,7 +675,6 @@ func (sa *Application) AddAllocationAsk(ask *AllocationAsk) error {
 		zap.String("ask", ask.GetAllocationKey()),
 		zap.Bool("placeholder", ask.IsPlaceholder()),
 		zap.Stringer("pendingDelta", delta))
-
 	sa.sortedRequests.insert(ask)
 	sa.appEvents.sendNewAskEvent(ask)
 
@@ -923,13 +906,16 @@ func (sa *Application) getOutstandingRequests(headRoom *resources.Resource, user
 		return
 	}
 	for _, request := range sa.sortedRequests {
-		if request.GetPendingAskRepeat() == 0 {
+		if request.GetPendingAskRepeat() == 0 || !request.IsSchedulingAttempted() {
 			continue
 		}
+
 		// ignore nil checks resource function calls are nil safe
 		if headRoom.FitInMaxUndef(request.GetAllocatedResource()) && userHeadRoom.FitInMaxUndef(request.GetAllocatedResource()) {
-			// if headroom is still enough for the resources
-			*total = append(*total, request)
+			if !request.HasTriggeredScaleUp() && request.requiredNode == common.Empty && !sa.canReplace(request) {
+				// if headroom is still enough for the resources
+				*total = append(*total, request)
+			}
 			headRoom.SubOnlyExisting(request.GetAllocatedResource())
 			userHeadRoom.SubOnlyExisting(request.GetAllocatedResource())
 		}
@@ -972,8 +958,12 @@ func (sa *Application) tryAllocate(headRoom *resources.Resource, allowPreemption
 		// NOTE: preemption most likely will not help in this case. The chance that preemption helps is mall
 		// as the preempted allocation must be for the same user in a different queue in the hierarchy...
 		if !userHeadroom.FitInMaxUndef(request.GetAllocatedResource()) {
+			request.LogAllocationFailure(NotEnoughUserQuota, true) // error message MUST be constant!
+			request.setUserQuotaCheckFailed(userHeadroom)
 			continue
 		}
+		request.setUserQuotaCheckPassed()
+		request.SetSchedulingAttempted(true)
 
 		// resource must fit in headroom otherwise skip the request (unless preemption could help)
 		if !headRoom.FitInMaxUndef(request.GetAllocatedResource()) {
@@ -988,10 +978,11 @@ func (sa *Application) tryAllocate(headRoom *resources.Resource, allowPreemption
 					}
 				}
 			}
-
-			sa.appEvents.sendAppDoesNotFitEvent(request)
+			request.LogAllocationFailure(NotEnoughQueueQuota, true) // error message MUST be constant!
+			request.setHeadroomCheckFailed(headRoom, sa.queuePath)
 			continue
 		}
+		request.setHeadroomCheckPassed(sa.queuePath)
 
 		requiredNode := request.GetRequiredNode()
 		// does request have any constraint to run on specific node?
@@ -999,7 +990,7 @@ func (sa *Application) tryAllocate(headRoom *resources.Resource, allowPreemption
 			// the iterator might not have the node we need as it could be reserved, or we have not added it yet
 			node := getNodeFn(requiredNode)
 			if node == nil {
-				rateLimitedLog.Warn("required node is not found (could be transient)",
+				getRateLimitedAppLog().Info("required node is not found (could be transient)",
 					zap.String("application ID", sa.ApplicationID),
 					zap.String("allocationKey", request.GetAllocationKey()),
 					zap.String("required node", requiredNode))
@@ -1140,7 +1131,7 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 			if delta.HasNegativeValue() {
 				log.Log(log.SchedApplication).Warn("releasing placeholder: real allocation is larger than placeholder",
 					zap.Stringer("requested resource", request.GetAllocatedResource()),
-					zap.String("placeholderID", ph.GetUUID()),
+					zap.String("placeholderID", ph.GetAllocationID()),
 					zap.Stringer("placeholder resource", ph.GetAllocatedResource()))
 				// release the placeholder and tell the RM
 				ph.SetReleased(true)
@@ -1158,7 +1149,7 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 			// got the node run same checks as for reservation (all but fits)
 			// resource usage should not change anyway between placeholder and real one at this point
 			if node != nil && node.preReserveConditions(request) {
-				alloc := NewAllocation(common.GetNewUUID(), node.NodeID, request)
+				alloc := NewAllocation(node.NodeID, request)
 				// double link to make it easier to find
 				// alloc (the real one) releases points to the placeholder in the releases list
 				alloc.SetRelease(ph)
@@ -1200,7 +1191,7 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 				return true
 			}
 			// allocation worked: on a non placeholder node update result and return
-			alloc := NewAllocation(common.GetNewUUID(), node.NodeID, reqFit)
+			alloc := NewAllocation(node.NodeID, reqFit)
 			// double link to make it easier to find
 			// alloc (the real one) releases points to the placeholder in the releases list
 			alloc.SetRelease(phFit)
@@ -1232,6 +1223,12 @@ func (sa *Application) tryPlaceholderAllocate(nodeIterator func() NodeIterator,
 	return allocResult
 }
 
+// check ask against both user headRoom and queue headRoom
+func (sa *Application) checkHeadRooms(ask *AllocationAsk, userHeadroom *resources.Resource, headRoom *resources.Resource) bool {
+	// check if this fits in the users' headroom first, if that fits check the queues' headroom
+	return userHeadroom.FitInMaxUndef(ask.GetAllocatedResource()) && headRoom.FitInMaxUndef(ask.GetAllocatedResource())
+}
+
 // Try a reserved allocation of an outstanding reservation
 func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIterator func() NodeIterator) *Allocation {
 	sa.Lock()
@@ -1259,13 +1256,8 @@ func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIte
 			alloc := newUnreservedAllocation(reserve.nodeID, unreserveAsk)
 			return alloc
 		}
-		// check if this fits in the users' headroom first, if that fits check the queues' headroom
-		if !userHeadroom.FitInMaxUndef(ask.GetAllocatedResource()) {
-			continue
-		}
 
-		// check if this fits in the queue's headroom
-		if !headRoom.FitInMaxUndef(ask.GetAllocatedResource()) {
+		if !sa.checkHeadRooms(ask, userHeadroom, headRoom) {
 			continue
 		}
 
@@ -1289,12 +1281,16 @@ func (sa *Application) tryReservedAllocate(headRoom *resources.Resource, nodeIte
 	// lets try this on all other nodes
 	for _, reserve := range sa.reservations {
 		// Other nodes cannot be tried if the ask has a required node
-		if reserve.ask.GetRequiredNode() != "" {
+		ask := reserve.ask
+		if ask.GetRequiredNode() != "" {
 			continue
 		}
 		iterator := nodeIterator()
 		if iterator != nil {
-			alloc := sa.tryNodesNoReserve(reserve.ask, iterator, reserve.nodeID)
+			if !sa.checkHeadRooms(ask, userHeadroom, headRoom) {
+				continue
+			}
+			alloc := sa.tryNodesNoReserve(ask, iterator, reserve.nodeID)
 			// have a candidate return it, including the node that was reserved
 			if alloc != nil {
 				return alloc
@@ -1496,13 +1492,13 @@ func (sa *Application) tryNode(node *Node, ask *AllocationAsk) *Allocation {
 	}
 
 	// everything OK really allocate
-	alloc := NewAllocation(common.GetNewUUID(), node.NodeID, ask)
+	alloc := NewAllocation(node.NodeID, ask)
 	if node.AddAllocation(alloc) {
 		if err := sa.queue.IncAllocatedResource(alloc.GetAllocatedResource(), false); err != nil {
-			log.Log(log.SchedApplication).Warn("queue update failed unexpectedly",
+			log.Log(log.SchedApplication).DPanic("queue update failed unexpectedly",
 				zap.Error(err))
 			// revert the node update
-			node.RemoveAllocation(alloc.GetUUID())
+			node.RemoveAllocation(alloc.GetAllocationID())
 			return nil
 		}
 		// mark this ask as allocated by lowering the repeat
@@ -1665,7 +1661,7 @@ func (sa *Application) addAllocationInternal(info *Allocation) {
 		sa.maxAllocatedResource = resources.ComponentWiseMax(sa.allocatedResource, sa.maxAllocatedResource)
 	}
 	sa.appEvents.sendNewAllocationEvent(info)
-	sa.allocations[info.GetUUID()] = info
+	sa.allocations[info.GetAllocationID()] = info
 }
 
 // Increase user resource usage
@@ -1682,9 +1678,12 @@ func (sa *Application) decUserResourceUsage(resource *resources.Resource, remove
 
 // Track used and preempted resources
 func (sa *Application) trackCompletedResource(info *Allocation) {
-	if info.IsPreempted() {
+	switch {
+	case info.IsPreempted():
 		sa.updatePreemptedResource(info)
-	} else {
+	case info.IsPlaceholder():
+		sa.updatePlaceholderResource(info)
+	default:
 		sa.updateUsedResource(info)
 	}
 }
@@ -1696,6 +1695,13 @@ func (sa *Application) updateUsedResource(info *Allocation) {
 		info.GetAllocatedResource(), info.GetBindTime())
 }
 
+// When the placeholder allocated with this allocation is to be removed,
+// have the placeholderResource to aggregate the resource used by this allocation
+func (sa *Application) updatePlaceholderResource(info *Allocation) {
+	sa.placeholderResource.AggregateTrackedResource(info.GetInstanceType(),
+		info.GetAllocatedResource(), info.GetBindTime())
+}
+
 // When the resource allocated with this allocation is to be preempted,
 // have the preemptedResource to aggregate the resource used by this allocation
 func (sa *Application) updatePreemptedResource(info *Allocation) {
@@ -1703,11 +1709,11 @@ func (sa *Application) updatePreemptedResource(info *Allocation) {
 		info.GetAllocatedResource(), info.GetBindTime())
 }
 
-func (sa *Application) ReplaceAllocation(uuid string) *Allocation {
+func (sa *Application) ReplaceAllocation(allocationID string) *Allocation {
 	sa.Lock()
 	defer sa.Unlock()
 	// remove the placeholder that was just confirmed by the shim
-	ph := sa.removeAllocationInternal(uuid, si.TerminationType_PLACEHOLDER_REPLACED)
+	ph := sa.removeAllocationInternal(allocationID, si.TerminationType_PLACEHOLDER_REPLACED)
 	// this has already been replaced or it is a duplicate message from the shim
 	if ph == nil || ph.GetReleaseCount() == 0 {
 		log.Log(log.SchedApplication).Debug("Unexpected placeholder released",
@@ -1719,7 +1725,7 @@ func (sa *Application) ReplaceAllocation(uuid string) *Allocation {
 	if ph.GetReleaseCount() > 1 {
 		log.Log(log.SchedApplication).Error("Unexpected release number, placeholder released, only 1 real allocations processed",
 			zap.String("applicationID", sa.ApplicationID),
-			zap.String("placeholderID", uuid),
+			zap.String("placeholderID", allocationID),
 			zap.Int("releases", ph.GetReleaseCount()))
 	}
 	// update the replacing allocation
@@ -1743,16 +1749,16 @@ func (sa *Application) ReplaceAllocation(uuid string) *Allocation {
 
 // Remove the Allocation from the application.
 // Return the allocation that was removed or nil if not found.
-func (sa *Application) RemoveAllocation(uuid string, releaseType si.TerminationType) *Allocation {
+func (sa *Application) RemoveAllocation(allocationID string, releaseType si.TerminationType) *Allocation {
 	sa.Lock()
 	defer sa.Unlock()
-	return sa.removeAllocationInternal(uuid, releaseType)
+	return sa.removeAllocationInternal(allocationID, releaseType)
 }
 
 // Remove the Allocation from the application
 // No locking must be called while holding the lock
-func (sa *Application) removeAllocationInternal(uuid string, releaseType si.TerminationType) *Allocation {
-	alloc := sa.allocations[uuid]
+func (sa *Application) removeAllocationInternal(allocationID string, releaseType si.TerminationType) *Allocation {
+	alloc := sa.allocations[allocationID]
 
 	// When app has the allocation, update map, and update allocated resource of the app
 	if alloc == nil {
@@ -1791,6 +1797,9 @@ func (sa *Application) removeAllocationInternal(uuid string, releaseType si.Term
 				eventWarning = "Application state not changed while removing a placeholder allocation"
 			}
 		}
+		// Aggregate the resources used by this alloc to the application's resource tracker
+		sa.trackCompletedResource(alloc)
+
 		sa.decUserResourceUsage(alloc.GetAllocatedResource(), removeApp)
 	} else {
 		sa.allocatedResource = resources.Sub(sa.allocatedResource, alloc.GetAllocatedResource())
@@ -1814,7 +1823,7 @@ func (sa *Application) removeAllocationInternal(uuid string, releaseType si.Term
 				zap.Error(err))
 		}
 	}
-	delete(sa.allocations, uuid)
+	delete(sa.allocations, allocationID)
 	sa.appEvents.sendRemoveAllocationEvent(alloc, releaseType)
 	return alloc
 }
@@ -1825,10 +1834,7 @@ func (sa *Application) updateAskMaxPriority() {
 		if v.GetPendingAskRepeat() == 0 {
 			continue
 		}
-		p := v.GetPriority()
-		if p > value {
-			value = p
-		}
+		value = max(value, v.GetPriority())
 	}
 	sa.askMaxPriority = value
 	sa.queue.UpdateApplicationPriority(sa.ApplicationID, value)
@@ -1852,7 +1858,9 @@ func (sa *Application) RemoveAllAllocations() []*Allocation {
 		sa.appEvents.sendRemoveAllocationEvent(alloc, si.TerminationType_STOPPED_BY_RM)
 	}
 
-	if resources.IsZero(sa.pending) {
+	// if an app doesn't have any allocations and the user doesn't have other applications,
+	// the user tracker is nonexistent. We don't want to decrease resource usage in this case.
+	if ugm.GetUserManager().GetUserTracker(sa.user.User) != nil && resources.IsZero(sa.pending) {
 		sa.decUserResourceUsage(resources.Add(sa.allocatedResource, sa.allocatedPlaceholder), true)
 	}
 	// cleanup allocated resource for app (placeholders and normal)
@@ -1944,7 +1952,7 @@ func (sa *Application) notifyRMAllocationReleased(rmID string, released []*Alloc
 			ApplicationID:   alloc.GetApplicationID(),
 			PartitionName:   alloc.GetPartitionName(),
 			AllocationKey:   alloc.GetAllocationKey(),
-			UUID:            alloc.GetUUID(),
+			AllocationID:    alloc.GetAllocationID(),
 			TerminationType: terminationType,
 			Message:         message,
 		})
@@ -1986,7 +1994,7 @@ func (sa *Application) notifyRMAllocationAskReleased(rmID string, released []*Al
 func (sa *Application) IsAllocationAssignedToApp(alloc *Allocation) bool {
 	sa.RLock()
 	defer sa.RUnlock()
-	_, ok := sa.allocations[alloc.GetUUID()]
+	_, ok := sa.allocations[alloc.GetAllocationID()]
 	return ok
 }
 
@@ -2033,6 +2041,7 @@ func (sa *Application) cleanupAsks() {
 
 func (sa *Application) cleanupTrackedResource() {
 	sa.usedResource = nil
+	sa.placeholderResource = nil
 	sa.preemptedResource = nil
 }
 
@@ -2049,6 +2058,8 @@ func (sa *Application) LogAppSummary(rmID string) {
 	appSummary := sa.GetApplicationSummary(rmID)
 	appSummary.DoLogging()
 	appSummary.ResourceUsage = nil
+	appSummary.PreemptedResource = nil
+	appSummary.PlaceholderResource = nil
 }
 
 func (sa *Application) HasPlaceholderAllocation() bool {
@@ -2057,12 +2068,12 @@ func (sa *Application) HasPlaceholderAllocation() bool {
 	return sa.hasPlaceholderAlloc
 }
 
-// test only
+// SetCompletingTimeout should be used for testing only.
 func SetCompletingTimeout(duration time.Duration) {
 	completingTimeout = duration
 }
 
-// test only
+// SetTimedOutPlaceholder should be used for testing only.
 func (sa *Application) SetTimedOutPlaceholder(taskGroupName string, timedOut int64) {
 	sa.Lock()
 	defer sa.Unlock()
@@ -2073,3 +2084,49 @@ func (sa *Application) SetTimedOutPlaceholder(taskGroupName string, timedOut int
 		sa.placeholderData[taskGroupName].TimedOut = timedOut
 	}
 }
+
+// getRateLimitedAppLog lazy initializes the application logger the first time is needed.
+func getRateLimitedAppLog() *log.RateLimitedLogger {
+	initAppLogOnce.Do(func() {
+		rateLimitedAppLog = log.NewRateLimitedLogger(log.SchedApplication, time.Second)
+	})
+	return rateLimitedAppLog
+}
+
+func (sa *Application) updateRunnableStatus(runnableInQueue, runnableByUserLimit bool) {
+	sa.Lock()
+	defer sa.Unlock()
+	if sa.runnableInQueue != runnableInQueue {
+		if runnableInQueue {
+			log.Log(log.SchedApplication).Info("Application is now runnable in queue",
+				zap.String("appID", sa.ApplicationID),
+				zap.String("queue", sa.queuePath))
+			sa.appEvents.sendAppRunnableInQueueEvent()
+		} else {
+			log.Log(log.SchedApplication).Info("Maximum number of running applications reached the queue limit",
+				zap.String("appID", sa.ApplicationID),
+				zap.String("queue", sa.queuePath))
+			sa.appEvents.sendAppNotRunnableInQueueEvent()
+		}
+	}
+	sa.runnableInQueue = runnableInQueue
+
+	if sa.runnableByUserLimit != runnableByUserLimit {
+		if runnableByUserLimit {
+			log.Log(log.SchedApplication).Info("Application is now runnable based on user/group quota",
+				zap.String("appID", sa.ApplicationID),
+				zap.String("queue", sa.queuePath),
+				zap.String("user", sa.user.User),
+				zap.Strings("groups", sa.user.Groups))
+			sa.appEvents.sendAppRunnableQuotaEvent()
+		} else {
+			log.Log(log.SchedApplication).Info("Maximum number of running applications reached the user/group limit",
+				zap.String("appID", sa.ApplicationID),
+				zap.String("queue", sa.queuePath),
+				zap.String("user", sa.user.User),
+				zap.Strings("groups", sa.user.Groups))
+			sa.appEvents.sendAppNotRunnableQuotaEvent()
+		}
+	}
+	sa.runnableByUserLimit = runnableByUserLimit
+}
