@@ -19,17 +19,14 @@
 package objects
 
 import (
-	"errors"
-	"fmt"
 	"testing"
 	"time"
 
 	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common/resources"
+	"github.com/apache/yunikorn-core/pkg/mock"
 	"github.com/apache/yunikorn-core/pkg/plugins"
-	"github.com/apache/yunikorn-scheduler-interface/lib/go/api"
-	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
 func TestCheckPreconditions(t *testing.T) {
@@ -107,8 +104,8 @@ func TestCheckPreemptionQueueGuarantees(t *testing.T) {
 	assert.NilError(t, app1.AddAllocationAsk(ask1))
 	ask2 := newAllocationAsk("alloc2", appID1, resources.NewResourceFromMap(map[string]resources.Quantity{"first": 5}))
 	assert.NilError(t, app1.AddAllocationAsk(ask2))
-	app1.AddAllocation(NewAllocation("alloc1", "node1", ask1))
-	app1.AddAllocation(NewAllocation("alloc2", "node1", ask2))
+	app1.AddAllocation(NewAllocation("node1", ask1))
+	app1.AddAllocation(NewAllocation("node1", ask2))
 	assert.NilError(t, childQ1.IncAllocatedResource(ask1.GetAllocatedResource(), false))
 	assert.NilError(t, childQ1.IncAllocatedResource(ask2.GetAllocatedResource(), false))
 	app2 := newApplication(appID2, "default", "root.parent.child2")
@@ -159,8 +156,8 @@ func TestCheckPreemptionQueueGuaranteesWithNoGuaranteedResources(t *testing.T) {
 			assert.NilError(t, app1.AddAllocationAsk(ask1))
 			ask2 := newAllocationAsk("alloc2", appID1, resources.NewResourceFromMap(map[string]resources.Quantity{"first": 5}))
 			assert.NilError(t, app1.AddAllocationAsk(ask2))
-			app1.AddAllocation(NewAllocation("alloc1", "node1", ask1))
-			app1.AddAllocation(NewAllocation("alloc2", "node1", ask2))
+			app1.AddAllocation(NewAllocation("node1", ask1))
+			app1.AddAllocation(NewAllocation("node1", ask2))
 			assert.NilError(t, childQ1.IncAllocatedResource(ask1.GetAllocatedResource(), false))
 			assert.NilError(t, childQ1.IncAllocatedResource(ask2.GetAllocatedResource(), false))
 			app2 := newApplication(appID2, "default", "root.parent.child2")
@@ -196,10 +193,10 @@ func TestTryPreemption(t *testing.T) {
 	ask2 := newAllocationAsk("alloc2", appID1, resources.NewResourceFromMap(map[string]resources.Quantity{"first": 5, "pods": 1}))
 	ask2.createTime = time.Now()
 	assert.NilError(t, app1.AddAllocationAsk(ask2))
-	alloc1 := NewAllocation("alloc1", "node1", ask1)
+	alloc1 := NewAllocation("node1", ask1)
 	app1.AddAllocation(alloc1)
 	assert.Check(t, node.AddAllocation(alloc1), "node alloc1 failed")
-	alloc2 := NewAllocation("alloc2", "node1", ask2)
+	alloc2 := NewAllocation("node1", ask2)
 	assert.Check(t, node.AddAllocation(alloc2), "node alloc2 failed")
 	node.AddAllocation(alloc2)
 	assert.NilError(t, childQ1.IncAllocatedResource(ask1.GetAllocatedResource(), false))
@@ -214,25 +211,15 @@ func TestTryPreemption(t *testing.T) {
 	preemptor := NewPreemptor(app2, headRoom, 30*time.Second, ask3, iterator(), false)
 
 	// register predicate handler
-	pluginErr := &errHolder{}
-	plugin := &mockPredicates{
-		errHolder:    pluginErr,
-		reservations: nil,
-		allocations:  nil,
-		preemptions: []mockPreemption{{
-			expectedAllocationKey:  "alloc3",
-			expectedNodeID:         "node1",
-			expectedAllocationKeys: []string{"alloc1"},
-			expectedStartIndex:     0,
-			success:                true,
-			index:                  0,
-		}},
+	preemptions := []mock.Preemption{
+		mock.NewPreemption(true, "alloc3", "node1", []string{"alloc1"}, 0, 0),
 	}
+	plugin := mock.NewPreemptionPredicatePlugin(nil, nil, preemptions)
 	plugins.RegisterSchedulerPlugin(plugin)
 	defer plugins.UnregisterSchedulerPlugins()
 
 	alloc, ok := preemptor.TryPreemption()
-	assert.NilError(t, pluginErr.err)
+	assert.NilError(t, plugin.GetPredicateError())
 	assert.Assert(t, ok, "no victims found")
 	assert.Equal(t, "alloc3", alloc.allocationKey, "wrong alloc")
 	assert.Check(t, alloc1.IsPreempted(), "alloc1 preempted")
@@ -288,110 +275,5 @@ func allocForScore(originator bool, allowPreemptSelf bool) *Allocation {
 	ask := NewAllocationAsk("alloc1", appID1, resources.NewResource())
 	ask.originator = originator
 	ask.allowPreemptSelf = allowPreemptSelf
-	return NewAllocation("alloc1", nodeID1, ask)
-}
-
-type mockPreemption struct {
-	expectedAllocationKey  string
-	expectedNodeID         string
-	expectedAllocationKeys []string
-	expectedStartIndex     int32
-	success                bool
-	index                  int32
-}
-
-var _ api.ResourceManagerCallback = &mockPredicates{}
-
-type errHolder struct {
-	err error
-}
-
-type mockPredicates struct {
-	reservations map[string]string
-	allocations  map[string]string
-	preemptions  []mockPreemption
-	errHolder    *errHolder
-	api.ResourceManagerCallback
-}
-
-func (m mockPredicates) Predicates(args *si.PredicatesArgs) error {
-	if args.Allocate {
-		nodeID, ok := m.allocations[args.AllocationKey]
-		if !ok {
-			return errors.New("no allocation found")
-		}
-		if nodeID != args.NodeID {
-			return errors.New("wrong node")
-		}
-		return nil
-	} else {
-		nodeID, ok := m.reservations[args.AllocationKey]
-		if !ok {
-			return errors.New("no allocation found")
-		}
-		if nodeID != args.NodeID {
-			return errors.New("wrong node")
-		}
-		return nil
-	}
-}
-
-func (m mockPredicates) PreemptionPredicates(args *si.PreemptionPredicatesArgs) *si.PreemptionPredicatesResponse {
-	result := &si.PreemptionPredicatesResponse{
-		Success: false,
-		Index:   -1,
-	}
-	for _, preemption := range m.preemptions {
-		if preemption.expectedAllocationKey != args.AllocationKey {
-			continue
-		}
-		if preemption.expectedNodeID != args.NodeID {
-			continue
-		}
-		if preemption.expectedStartIndex != args.StartIndex {
-			m.errHolder.err = fmt.Errorf("unexpected start index exepected=%d, actual=%d, allocationKey=%s",
-				preemption.expectedStartIndex, args.StartIndex, args.AllocationKey)
-			return result
-		}
-		if len(preemption.expectedAllocationKeys) != len(args.PreemptAllocationKeys) {
-			m.errHolder.err = fmt.Errorf("unexpected alloc key length expected=%d, actual=%d, allocationKey=%s",
-				len(preemption.expectedAllocationKeys), len(args.PreemptAllocationKeys), args.AllocationKey)
-			return result
-		}
-		for idx, key := range preemption.expectedAllocationKeys {
-			if args.PreemptAllocationKeys[idx] != key {
-				m.errHolder.err = fmt.Errorf("unexpected preempt alloc key expected=%s, actual=%s, index=%d, allocationKey=%s",
-					args.PreemptAllocationKeys[idx], key, idx, args.AllocationKey)
-				return result
-			}
-		}
-		result.Success = preemption.success
-		result.Index = preemption.index
-		return result
-	}
-	m.errHolder.err = fmt.Errorf("mo match found allocationKey=%s, nodeID=%s", args.AllocationKey, args.NodeID)
-	return result
-}
-
-func (m mockPredicates) UpdateAllocation(_ *si.AllocationResponse) error {
-	// no implementation
-	return nil
-}
-
-func (m mockPredicates) UpdateApplication(_ *si.ApplicationResponse) error {
-	// no implementation
-	return nil
-}
-
-func (m mockPredicates) UpdateNode(_ *si.NodeResponse) error {
-	// no implementation
-	return nil
-}
-
-func (m mockPredicates) SendEvent(_ []*si.EventRecord) {
-	// no implementation
-}
-
-func (m mockPredicates) UpdateContainerSchedulingState(_ *si.UpdateContainerSchedulingStateRequest) {
-	// no implementation
+	return NewAllocation(nodeID1, ask)
 }
