@@ -208,6 +208,7 @@ func newDynamicQueueInternal(name string, leaf bool, parent *Queue) (*Queue, err
 
 // applyTemplate uses input template to initialize properties, maxResource, and guaranteedResource
 func (sq *Queue) applyTemplate(childTemplate *template.Template) {
+	sq.maxRunningApps = childTemplate.GetMaxApplications()
 	sq.properties = childTemplate.GetProperties()
 	// the resources in template are already checked
 	sq.guaranteedResource = childTemplate.GetGuaranteedResource()
@@ -360,6 +361,7 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 	maxResource, err := resources.NewResourceFromConf(resource.Max)
 	if err != nil {
 		log.Log(log.SchedQueue).Error("parsing failed on max resources this should not happen",
+			zap.String("queue", sq.QueuePath),
 			zap.Error(err))
 		return err
 	}
@@ -368,30 +370,62 @@ func (sq *Queue) setResources(resource configs.Resources) error {
 	guaranteedResource, err = resources.NewResourceFromConf(resource.Guaranteed)
 	if err != nil {
 		log.Log(log.SchedQueue).Error("parsing failed on guaranteed resources this should not happen",
+			zap.String("queue", sq.QueuePath),
 			zap.Error(err))
 		return err
 	}
 
-	if resources.StrictlyGreaterThanZero(maxResource) {
+	switch {
+	case resources.StrictlyGreaterThanZero(maxResource):
+		log.Log(log.SchedQueue).Debug("setting max resources",
+			zap.String("queue", sq.QueuePath),
+			zap.Stringer("current", sq.maxResource),
+			zap.Stringer("new", maxResource))
 		if !resources.Equals(sq.maxResource, maxResource) && sq.queueEvents != nil {
 			sq.queueEvents.sendMaxResourceChangedEvent()
 		}
 		sq.maxResource = maxResource
 		sq.updateMaxResourceMetrics()
-	} else {
-		log.Log(log.SchedQueue).Debug("max resources setting ignored: cannot set zero max resources")
+	case sq.maxResource != nil:
+		log.Log(log.SchedQueue).Debug("setting max resources",
+			zap.String("queue", sq.QueuePath),
+			zap.Stringer("current", sq.maxResource),
+			zap.Stringer("new", maxResource))
+		if sq.queueEvents != nil {
+			sq.queueEvents.sendMaxResourceChangedEvent()
+		}
+		sq.maxResource = nil
+		sq.updateMaxResourceMetrics()
+	default:
+		log.Log(log.SchedQueue).Debug("max resources setting ignored: cannot set zero max resources",
+			zap.String("queue", sq.QueuePath))
 	}
 
-	if resources.StrictlyGreaterThanZero(guaranteedResource) {
+	switch {
+	case resources.StrictlyGreaterThanZero(guaranteedResource):
+		log.Log(log.SchedQueue).Debug("setting guaranteed resources",
+			zap.String("queue", sq.QueuePath),
+			zap.Stringer("current", sq.guaranteedResource),
+			zap.Stringer("new", guaranteedResource))
 		if !resources.Equals(sq.guaranteedResource, guaranteedResource) && sq.queueEvents != nil {
 			sq.queueEvents.sendGuaranteedResourceChangedEvent()
 		}
 		sq.guaranteedResource = guaranteedResource
 		sq.updateGuaranteedResourceMetrics()
-	} else {
-		log.Log(log.SchedQueue).Debug("guaranteed resources setting ignored: cannot set zero max resources")
+	case sq.guaranteedResource != nil:
+		log.Log(log.SchedQueue).Debug("setting guaranteed resources",
+			zap.String("queue", sq.QueuePath),
+			zap.Stringer("current", sq.guaranteedResource),
+			zap.Stringer("new", guaranteedResource))
+		if sq.queueEvents != nil {
+			sq.queueEvents.sendGuaranteedResourceChangedEvent()
+		}
+		sq.guaranteedResource = nil
+		sq.updateGuaranteedResourceMetrics()
+	default:
+		log.Log(log.SchedQueue).Debug("guaranteed resources setting ignored: cannot set zero guaranteed resources",
+			zap.String("queue", sq.QueuePath))
 	}
-
 	return nil
 }
 
@@ -590,17 +624,23 @@ func (sq *Queue) CheckAdminAccess(user security.UserGroup) bool {
 }
 
 // GetPartitionQueueDAOInfo returns the queue hierarchy as an object for a REST call.
-func (sq *Queue) GetPartitionQueueDAOInfo() dao.PartitionQueueDAOInfo {
+// Exclude is true, which means that returns the specified queue object, but does not return the children of the specified queue.
+func (sq *Queue) GetPartitionQueueDAOInfo(exclude bool) dao.PartitionQueueDAOInfo {
 	queueInfo := dao.PartitionQueueDAOInfo{}
-	childes := sq.GetCopyOfChildren()
-	queueInfo.Children = make([]dao.PartitionQueueDAOInfo, 0, len(childes))
-	for _, child := range childes {
-		queueInfo.Children = append(queueInfo.Children, child.GetPartitionQueueDAOInfo())
+	children := sq.GetCopyOfChildren()
+	if !exclude {
+		queueInfo.Children = make([]dao.PartitionQueueDAOInfo, 0, len(children))
+		for _, child := range children {
+			queueInfo.Children = append(queueInfo.Children, child.GetPartitionQueueDAOInfo(false))
+		}
 	}
 	// we have held the read lock so following method should not take lock again.
 	sq.RLock()
 	defer sq.RUnlock()
 
+	for _, child := range children {
+		queueInfo.ChildrenNames = append(queueInfo.ChildrenNames, child.QueuePath)
+	}
 	queueInfo.QueueName = sq.QueuePath
 	queueInfo.Status = sq.stateMachine.Current()
 	queueInfo.PendingResource = sq.pending.DAOMap()
@@ -608,6 +648,7 @@ func (sq *Queue) GetPartitionQueueDAOInfo() dao.PartitionQueueDAOInfo {
 	queueInfo.GuaranteedResource = sq.guaranteedResource.DAOMap()
 	queueInfo.AllocatedResource = sq.allocatedResource.DAOMap()
 	queueInfo.PreemptingResource = sq.preemptingResource.DAOMap()
+	queueInfo.HeadRoom = sq.getHeadRoom().DAOMap()
 	queueInfo.IsLeaf = sq.isLeaf
 	queueInfo.IsManaged = sq.isManaged
 	queueInfo.CurrentPriority = sq.getCurrentPriority()
@@ -884,23 +925,13 @@ func (sq *Queue) addChildQueue(child *Queue) error {
 		if child.isManaged {
 			return nil
 		}
-		// this is a story about compatibility. the template is a new feature, and we want to keep old behavior.
-		// 1) try to use template if it is not nil
-		// 2) otherwise, child leaf copy the configs.ApplicationSortPolicy from parent (old behavior)
+		// try to use template if it is not nil
 		if sq.template != nil {
 			log.Log(log.SchedQueue).Debug("applying child template to new leaf queue",
 				zap.String("child queue", child.QueuePath),
 				zap.String("parent queue", sq.QueuePath),
 				zap.Any("template", sq.template))
 			child.applyTemplate(sq.template)
-		} else {
-			policyValue, ok := sq.properties[configs.ApplicationSortPolicy]
-			if ok {
-				log.Log(log.Deprecation).Warn("inheriting application sort policy is deprecated, use child templates",
-					zap.String("child queue", child.QueuePath),
-					zap.String("parent queue", sq.QueuePath))
-				child.properties[configs.ApplicationSortPolicy] = policyValue
-			}
 		}
 		return nil
 	}
@@ -1034,7 +1065,7 @@ func (sq *Queue) DecAllocatedResource(alloc *resources.Resource) error {
 	defer sq.Unlock()
 
 	// check this queue: failure stops checks
-	if alloc != nil && !resources.FitIn(sq.allocatedResource, alloc) {
+	if alloc != nil && !sq.allocatedResource.FitIn(alloc) {
 		return fmt.Errorf("released allocation (%v) is larger than '%s' queue allocation (%v)",
 			alloc, sq.QueuePath, sq.allocatedResource)
 	}
@@ -1265,8 +1296,32 @@ func (sq *Queue) SetMaxResource(max *resources.Resource) {
 	log.Log(log.SchedQueue).Info("updating root queue max resources",
 		zap.Stringer("current max", sq.maxResource),
 		zap.Stringer("new max", max))
-	sq.maxResource = max.Clone()
-	sq.updateMaxResourceMetrics()
+
+	switch {
+	case resources.StrictlyGreaterThanZero(max):
+		log.Log(log.SchedQueue).Debug("setting max resources",
+			zap.String("queue", sq.QueuePath),
+			zap.Stringer("current", sq.maxResource),
+			zap.Stringer("new", max))
+		if !resources.Equals(sq.maxResource, max) && sq.queueEvents != nil {
+			sq.queueEvents.sendMaxResourceChangedEvent()
+		}
+		sq.maxResource = max.Clone()
+		sq.updateMaxResourceMetrics()
+	case sq.maxResource != nil:
+		log.Log(log.SchedQueue).Debug("setting max resources",
+			zap.String("queue", sq.QueuePath),
+			zap.Stringer("current", sq.maxResource),
+			zap.Stringer("new", max))
+		if sq.queueEvents != nil {
+			sq.queueEvents.sendMaxResourceChangedEvent()
+		}
+		sq.maxResource = nil
+		sq.updateMaxResourceMetrics()
+	default:
+		log.Log(log.SchedQueue).Debug("max resources setting ignored: cannot set zero max resources",
+			zap.String("queue", sq.QueuePath))
+	}
 }
 
 // canRunApp returns if the queue could run a new app for this queue (recursively).
@@ -1306,12 +1361,15 @@ func (sq *Queue) TryAllocate(iterator func() NodeIterator, fullIterator func() N
 
 		// process the apps (filters out app without pending requests)
 		for _, app := range sq.sortApplications(true, false) {
-			if app.IsAccepted() && (!sq.canRunApp(app.ApplicationID) || !ugm.GetUserManager().CanRunApp(sq.QueuePath, app.ApplicationID, app.user)) {
+			runnableInQueue := sq.canRunApp(app.ApplicationID)
+			runnableByUserLimit := ugm.GetUserManager().CanRunApp(sq.QueuePath, app.ApplicationID, app.user)
+			app.updateRunnableStatus(runnableInQueue, runnableByUserLimit)
+			if app.IsAccepted() && (!runnableInQueue || !runnableByUserLimit) {
 				continue
 			}
 			alloc := app.tryAllocate(headRoom, allowPreemption, preemptionDelay, &preemptAttemptsRemaining, iterator, fullIterator, getnode)
 			if alloc != nil {
-				log.Log(log.SchedQueue).Debug("allocation found on queue",
+				log.Log(log.SchedQueue).Info("allocation found on queue",
 					zap.String("queueName", sq.QueuePath),
 					zap.String("appID", app.ApplicationID),
 					zap.Stringer("allocation", alloc))
@@ -1347,7 +1405,7 @@ func (sq *Queue) TryPlaceholderAllocate(iterator func() NodeIterator, getnode fu
 		for _, app := range sq.sortApplications(true, true) {
 			alloc := app.tryPlaceholderAllocate(iterator, getnode)
 			if alloc != nil {
-				log.Log(log.SchedQueue).Debug("allocation found on queue",
+				log.Log(log.SchedQueue).Info("allocation found on queue",
 					zap.String("queueName", sq.QueuePath),
 					zap.String("appID", app.ApplicationID),
 					zap.Stringer("allocation", alloc))
@@ -1418,7 +1476,7 @@ func (sq *Queue) TryReservedAllocate(iterator func() NodeIterator) *Allocation {
 				}
 				alloc := app.tryReservedAllocate(headRoom, iterator)
 				if alloc != nil {
-					log.Log(log.SchedQueue).Debug("reservation found for allocation found on queue",
+					log.Log(log.SchedQueue).Info("reservation found for allocation found on queue",
 						zap.String("queueName", sq.QueuePath),
 						zap.String("appID", appID),
 						zap.Stringer("allocation", alloc),
@@ -1907,9 +1965,7 @@ func (sq *Queue) recalculatePriority() int32 {
 
 	curr := configs.MinPriority
 	for _, v := range items {
-		if v > curr {
-			curr = v
-		}
+		curr = max(v, curr)
 	}
 	sq.currentPriority = curr
 	return priorityValueByPolicy(sq.priorityPolicy, sq.priorityOffset, curr)
