@@ -26,11 +26,12 @@ import (
 	"gotest.tools/v3/assert"
 
 	"github.com/apache/yunikorn-core/pkg/common"
+	"github.com/apache/yunikorn-core/pkg/mock"
 	"github.com/apache/yunikorn-scheduler-interface/lib/go/si"
 )
 
 type mockRMCallback struct {
-	MockResourceManagerCallback
+	mock.ResourceManagerCallback
 	acceptedApplications map[string]bool
 	rejectedApplications map[string]bool
 	acceptedNodes        map[string]bool
@@ -41,8 +42,6 @@ type mockRMCallback struct {
 	sync.RWMutex
 }
 
-// This is only exported to allow the use in the simple_example.go.
-// Lint exclusion added as the non-export return is OK
 func newMockRMCallbackHandler() *mockRMCallback {
 	return &mockRMCallback{
 		acceptedApplications: make(map[string]bool),
@@ -72,7 +71,7 @@ func (m *mockRMCallback) UpdateAllocation(response *si.AllocationResponse) error
 	m.Lock()
 	defer m.Unlock()
 	for _, alloc := range response.New {
-		m.Allocations[alloc.UUID] = alloc
+		m.Allocations[alloc.AllocationID] = alloc
 		if val, ok := m.nodeAllocations[alloc.NodeID]; ok {
 			val = append(val, alloc)
 			m.nodeAllocations[alloc.NodeID] = val
@@ -83,7 +82,7 @@ func (m *mockRMCallback) UpdateAllocation(response *si.AllocationResponse) error
 		}
 	}
 	for _, alloc := range response.Released {
-		delete(m.Allocations, alloc.UUID)
+		delete(m.Allocations, alloc.AllocationID)
 	}
 	return nil
 }
