@@ -36,27 +36,42 @@ type QueueTracker struct {
 	maxResources        *resources.Resource
 	maxRunningApps      uint64
 	childQueueTrackers  map[string]*QueueTracker
+	useWildCard         bool
 }
 
-func newRootQueueTracker() *QueueTracker {
-	qt := newQueueTracker(common.Empty, configs.RootQueue)
+func newRootQueueTracker(trackType trackingType) *QueueTracker {
+	qt := newQueueTracker(common.Empty, configs.RootQueue, trackType)
 	return qt
 }
 
-func newQueueTracker(queuePath string, queueName string) *QueueTracker {
-	qp := queueName
+func newQueueTracker(queuePath string, queueName string, trackType trackingType) *QueueTracker {
+	fullPath := queueName
 	if queuePath != common.Empty {
-		qp = queuePath + "." + queueName
+		fullPath = queuePath + "." + queueName
 	}
 	queueTracker := &QueueTracker{
 		queueName:           queueName,
-		queuePath:           qp,
+		queuePath:           fullPath,
 		resourceUsage:       nil,
 		runningApplications: make(map[string]bool),
 		maxResources:        nil,
 		maxRunningApps:      0,
 		childQueueTrackers:  make(map[string]*QueueTracker),
 	}
+
+	// Override user/group specific limits with wild card limit settings
+	if trackType == user {
+		if config := m.getUserWildCardLimitsConfig(fullPath); config != nil {
+			log.Log(log.SchedUGM).Debug("Use wild card limit settings as there is no limit set explicitly",
+				zap.String("queue name", queueName),
+				zap.String("queue path", queuePath),
+				zap.Uint64("max applications", config.maxApplications),
+				zap.Stringer("max resources", config.maxResources))
+			queueTracker.maxResources = config.maxResources.Clone()
+			queueTracker.maxRunningApps = config.maxApplications
+			queueTracker.useWildCard = true
+		}
+	}
 	log.Log(log.SchedUGM).Debug("Created queue tracker object for queue",
 		zap.String("queue", queueName))
 	return queueTracker
@@ -76,93 +91,32 @@ func (qt *QueueTracker) increaseTrackedResource(hierarchy []string, applicationI
 		zap.String("queue path", qt.queuePath),
 		zap.Strings("hierarchy", hierarchy),
 		zap.String("application", applicationID),
-		zap.Stringer("resource", usage))
+		zap.Stringer("resource", usage),
+		zap.Bool("use wild card", qt.useWildCard))
 	// depth first: all the way to the leaf, create if not exists
 	// more than 1 in the slice means we need to recurse down
 	if len(hierarchy) > 1 {
 		childName := hierarchy[1]
 		if qt.childQueueTrackers[childName] == nil {
-			qt.childQueueTrackers[childName] = newQueueTracker(qt.queuePath, childName)
+			qt.childQueueTrackers[childName] = newQueueTracker(qt.queuePath, childName, trackType)
 		}
 		if !qt.childQueueTrackers[childName].increaseTrackedResource(hierarchy[1:], applicationID, trackType, usage) {
 			return false
 		}
 	}
-
 	if qt.resourceUsage == nil {
 		qt.resourceUsage = resources.NewResource()
 	}
-	finalResourceUsage := qt.resourceUsage.Clone()
-	finalResourceUsage.AddTo(usage)
-	wildCardQuotaExceeded := false
-	existingApp := qt.runningApplications[applicationID]
-
-	// apply user/group specific limit settings set if configured, otherwise use wild card limit settings
-	if qt.maxRunningApps != 0 && !resources.IsZero(qt.maxResources) {
-		log.Log(log.SchedUGM).Debug("applying enforcement checks using limit settings of specific user/group",
-			zap.Int("tracking type", int(trackType)),
-			zap.String("queue path", qt.queuePath),
-			zap.Bool("existing app", existingApp),
-			zap.Uint64("max running apps", qt.maxRunningApps),
-			zap.Stringer("max resources", qt.maxResources))
-		if (!existingApp && len(qt.runningApplications)+1 > int(qt.maxRunningApps)) ||
-			resources.StrictlyGreaterThan(finalResourceUsage, qt.maxResources) {
-			log.Log(log.SchedUGM).Warn("Unable to increase resource usage as allowing new application to run would exceed either configured max applications or max resources limit of specific user/group",
-				zap.Int("tracking type", int(trackType)),
-				zap.String("queue path", qt.queuePath),
-				zap.Bool("existing app", existingApp),
-				zap.Int("current running applications", len(qt.runningApplications)),
-				zap.Uint64("max running applications", qt.maxRunningApps),
-				zap.Stringer("current resource usage", qt.resourceUsage),
-				zap.Stringer("max resource usage", qt.maxResources))
-			return false
-		}
-	}
-
-	// Try wild card settings
-	if qt.maxRunningApps == 0 && resources.IsZero(qt.maxResources) {
-		// Is there any wild card settings? Do we need to apply enforcement checks using wild card limit settings?
-		var config *LimitConfig
-		if trackType == user {
-			config = m.getUserWildCardLimitsConfig(qt.queuePath)
-		} else if trackType == group {
-			config = m.getGroupWildCardLimitsConfig(qt.queuePath)
-		}
-		if config != nil {
-			wildCardQuotaExceeded = (config.maxApplications != 0 && !existingApp && len(qt.runningApplications)+1 > int(config.maxApplications)) ||
-				(!resources.IsZero(config.maxResources) && resources.StrictlyGreaterThan(finalResourceUsage, config.maxResources))
-			log.Log(log.SchedUGM).Debug("applying enforcement checks using limit settings of wild card user/group",
-				zap.Int("tracking type", int(trackType)),
-				zap.String("queue path", qt.queuePath),
-				zap.Bool("existing app", existingApp),
-				zap.Uint64("wild card max running apps", config.maxApplications),
-				zap.Stringer("wild card max resources", config.maxResources),
-				zap.Bool("wild card quota exceeded", wildCardQuotaExceeded))
-			if wildCardQuotaExceeded {
-				log.Log(log.SchedUGM).Warn("Unable to increase resource usage as allowing new application to run would exceed either configured max applications or max resources limit of wild card user/group",
-					zap.Int("tracking type", int(trackType)),
-					zap.String("queue path", qt.queuePath),
-					zap.Bool("existing app", existingApp),
-					zap.Int("current running applications", len(qt.runningApplications)),
-					zap.Uint64("max running applications", config.maxApplications),
-					zap.Stringer("current resource usage", qt.resourceUsage),
-					zap.Stringer("max resource usage", config.maxResources))
-				return false
-			}
-		}
-	}
-
 	qt.resourceUsage.AddTo(usage)
 	qt.runningApplications[applicationID] = true
-
 	log.Log(log.SchedUGM).Debug("Successfully increased resource usage",
 		zap.Int("tracking type", int(trackType)),
 		zap.String("queue path", qt.queuePath),
 		zap.String("application", applicationID),
-		zap.Bool("existing app", existingApp),
 		zap.Stringer("resource", usage),
 		zap.Uint64("max running applications", qt.maxRunningApps),
 		zap.Stringer("max resource usage", qt.maxResources),
+		zap.Bool("use wild card", qt.useWildCard),
 		zap.Stringer("total resource after increasing", qt.resourceUsage),
 		zap.Int("total applications after increasing", len(qt.runningApplications)))
 	return true
@@ -220,39 +174,40 @@ func (qt *QueueTracker) decreaseTrackedResource(hierarchy []string, applicationI
 	return removeQT, true
 }
 
-func (qt *QueueTracker) setLimit(hierarchy []string, maxResource *resources.Resource, maxApps uint64) {
+func (qt *QueueTracker) setLimit(hierarchy []string, maxResource *resources.Resource, maxApps uint64, useWildCard bool, trackType trackingType, doWildCardCheck bool) {
 	log.Log(log.SchedUGM).Debug("Setting limits",
 		zap.String("queue path", qt.queuePath),
 		zap.Strings("hierarchy", hierarchy),
 		zap.Uint64("max applications", maxApps),
-		zap.Stringer("max resources", maxResource))
+		zap.Stringer("max resources", maxResource),
+		zap.Bool("use wild card", useWildCard))
 	// depth first: all the way to the leaf, create if not exists
 	// more than 1 in the slice means we need to recurse down
 	if len(hierarchy) > 1 {
 		childName := hierarchy[1]
 		if qt.childQueueTrackers[childName] == nil {
-			qt.childQueueTrackers[childName] = newQueueTracker(qt.queuePath, childName)
+			qt.childQueueTrackers[childName] = newQueueTracker(qt.queuePath, childName, trackType)
 		}
-		qt.childQueueTrackers[childName].setLimit(hierarchy[1:], maxResource, maxApps)
+		qt.childQueueTrackers[childName].setLimit(hierarchy[1:], maxResource, maxApps, useWildCard, trackType, false)
 	} else if len(hierarchy) == 1 {
+		// don't override named user/group specific limits with wild card limits
+		if doWildCardCheck && !qt.useWildCard {
+			return
+		}
 		qt.maxRunningApps = maxApps
 		qt.maxResources = maxResource
+		qt.useWildCard = useWildCard
 	}
 }
 
 func (qt *QueueTracker) headroom(hierarchy []string, trackType trackingType) *resources.Resource {
-	log.Log(log.SchedUGM).Debug("Calculating headroom",
-		zap.String("queue path", qt.queuePath),
-		zap.Strings("hierarchy", hierarchy),
-		zap.Int("tracking type", int(trackType)),
-	)
 	// depth first: all the way to the leaf, create if not exists
 	// more than 1 in the slice means we need to recurse down
 	var headroom, childHeadroom *resources.Resource
 	if len(hierarchy) > 1 {
 		childName := hierarchy[1]
 		if qt.childQueueTrackers[childName] == nil {
-			qt.childQueueTrackers[childName] = newQueueTracker(qt.queuePath, childName)
+			qt.childQueueTrackers[childName] = newQueueTracker(qt.queuePath, childName, trackType)
 		}
 		childHeadroom = qt.childQueueTrackers[childName].headroom(hierarchy[1:], trackType)
 	}
@@ -261,30 +216,8 @@ func (qt *QueueTracker) headroom(hierarchy []string, trackType trackingType) *re
 	if !resources.IsZero(qt.maxResources) {
 		headroom = qt.maxResources.Clone()
 		headroom.SubOnlyExisting(qt.resourceUsage)
-		log.Log(log.SchedUGM).Debug("Calculated headroom",
-			zap.String("queue path", qt.queuePath),
-			zap.Int("tracking type", int(trackType)),
-			zap.Stringer("max resource", qt.maxResources),
-			zap.Stringer("headroom", headroom))
-	} else if resources.IsZero(childHeadroom) {
-		// If childHeadroom is not nil, it means there is an user or wildcard limit config in child queue,
-		// so we don't check wildcard limit config in current queue.
-
-		// Fall back on wild card user limit settings to calculate headroom only for unnamed users.
-		// For unnamed groups, "*" group tracker object would be used using the above block to calculate headroom
-		// because resource usage added together for all unnamed groups under "*" group tracker object.
-		if trackType == user {
-			if config := m.getUserWildCardLimitsConfig(qt.queuePath); config != nil {
-				headroom = config.maxResources.Clone()
-				headroom.SubOnlyExisting(qt.resourceUsage)
-				log.Log(log.SchedUGM).Debug("Calculated headroom",
-					zap.String("queue path", qt.queuePath),
-					zap.Int("tracking type", int(trackType)),
-					zap.Stringer("wildcard max resource", config.maxResources),
-					zap.Stringer("headroom", headroom))
-			}
-		}
 	}
+
 	if headroom == nil {
 		return childHeadroom
 	}
@@ -427,18 +360,13 @@ func (qt *QueueTracker) decreaseTrackedResourceUsageDownwards(hierarchy []string
 }
 
 func (qt *QueueTracker) canRunApp(hierarchy []string, applicationID string, trackType trackingType) bool {
-	log.Log(log.SchedUGM).Debug("Checking can run app",
-		zap.Int("tracking type", int(trackType)),
-		zap.String("queue path", qt.queuePath),
-		zap.String("application", applicationID),
-		zap.Strings("hierarchy", hierarchy))
 	// depth first: all the way to the leaf, create if not exists
 	// more than 1 in the slice means we need to recurse down
 	childCanRunApp := true
 	if len(hierarchy) > 1 {
 		childName := hierarchy[1]
 		if qt.childQueueTrackers[childName] == nil {
-			qt.childQueueTrackers[childName] = newQueueTracker(qt.queuePath, childName)
+			qt.childQueueTrackers[childName] = newQueueTracker(qt.queuePath, childName, trackType)
 		}
 		childCanRunApp = qt.childQueueTrackers[childName].canRunApp(hierarchy[1:], applicationID, trackType)
 	}
@@ -457,31 +385,8 @@ func (qt *QueueTracker) canRunApp(hierarchy []string, applicationID string, trac
 
 	// apply user/group specific limit settings set if configured, otherwise use wild card limit settings
 	if qt.maxRunningApps != 0 && running > int(qt.maxRunningApps) {
-		log.Log(log.SchedUGM).Warn("can't run app as allowing new application to run would exceed configured max applications limit of specific user/group",
-			zap.Int("tracking type", int(trackType)),
-			zap.String("queue path", qt.queuePath),
-			zap.Int("current running applications", len(qt.runningApplications)),
-			zap.Uint64("max running applications", qt.maxRunningApps))
 		return false
 	}
-
-	// Try wild card settings
-	if qt.maxRunningApps == 0 {
-		var config *LimitConfig
-		if trackType == user {
-			config = m.getUserWildCardLimitsConfig(qt.queuePath)
-		} else if trackType == group {
-			config = m.getGroupWildCardLimitsConfig(qt.queuePath)
-		}
-		if config != nil && config.maxApplications != 0 && running > int(config.maxApplications) {
-			log.Log(log.SchedUGM).Warn("can't run app as allowing new application to run would exceed configured max applications limit of wildcard user/group",
-				zap.Int("tracking type", int(trackType)),
-				zap.String("queue path", qt.queuePath),
-				zap.Int("current running applications", len(qt.runningApplications)),
-				zap.Uint64("max running applications", config.maxApplications))
-			return false
-		}
-	}
 	return true
 }
 
