@@ -20,8 +20,8 @@ package webservice
 
 import (
 	"context"
+	"errors"
 	"net/http"
-	"sync"
 	"time"
 
 	"github.com/julienschmidt/httprouter"
@@ -34,7 +34,6 @@ import (
 )
 
 var imHistory *history.InternalMetricsHistory
-var lock sync.RWMutex
 var schedulerContext *scheduler.ClusterContext
 
 type WebService struct {
@@ -54,7 +53,7 @@ func loggingHandler(inner http.Handler, name string) http.HandlerFunc {
 	return func(w http.ResponseWriter, r *http.Request) {
 		start := time.Now()
 		inner.ServeHTTP(w, r)
-		log.Logger().Debug("Web router call details",
+		log.Log(log.REST).Debug("Web router call details",
 			zap.String("name", name),
 			zap.String("method", r.Method),
 			zap.String("uri", r.RequestURI),
@@ -62,16 +61,17 @@ func loggingHandler(inner http.Handler, name string) http.HandlerFunc {
 	}
 }
 
+// StartWebApp starts the web app on the default port.
 // TODO we need the port to be configurable
 func (m *WebService) StartWebApp() {
 	router := newRouter()
 	m.httpServer = &http.Server{Addr: ":9080", Handler: router}
 
-	log.Logger().Info("web-app started", zap.Int("port", 9080))
+	log.Log(log.REST).Info("web-app started", zap.Int("port", 9080))
 	go func() {
 		httpError := m.httpServer.ListenAndServe()
-		if httpError != nil && httpError != http.ErrServerClosed {
-			log.Logger().Error("HTTP serving error",
+		if httpError != nil && !errors.Is(httpError, http.ErrServerClosed) {
+			log.Log(log.REST).Error("HTTP serving error",
 				zap.Error(httpError))
 		}
 	}()
