@@ -671,22 +671,22 @@ func TestUpdateApplicationReservationEventGetApplicationID(t *testing.T) {
 
 func TestNewReleaseAppAllocationEvent(t *testing.T) {
 	tests := []struct {
-		name                       string
-		appID, allocationUUID      string
-		terminationType            si.TerminationType
-		wantID, wantUUID, wantType string
-		wantEvent                  ApplicationEventType
+		name                               string
+		appID, allocationID                string
+		terminationType                    si.TerminationType
+		wantID, wantAllocationID, wantType string
+		wantEvent                          ApplicationEventType
 	}{
-		{TestCreateName, "testAppId001", "testUUID001", si.TerminationType_TIMEOUT, "testAppId001", "testUUID001", "TIMEOUT", ReleaseAppAllocation},
+		{TestCreateName, "testAppId001", "testAllocationID001", si.TerminationType_TIMEOUT, "testAppId001", "testAllocationID001", "TIMEOUT", ReleaseAppAllocation},
 	}
 
 	for _, tt := range tests {
-		instance := NewReleaseAppAllocationEvent(tt.appID, tt.terminationType, tt.allocationUUID)
+		instance := NewReleaseAppAllocationEvent(tt.appID, tt.terminationType, tt.allocationID)
 		t.Run(tt.name, func(t *testing.T) {
-			if instance.applicationID != tt.wantID || instance.allocationUUID != tt.wantUUID || instance.terminationType != tt.wantType || instance.event != tt.wantEvent {
+			if instance.applicationID != tt.wantID || instance.allocationID != tt.wantAllocationID || instance.terminationType != tt.wantType || instance.event != tt.wantEvent {
 				t.Errorf("want %s %s %s %s, got %s %s %s %s",
-					tt.wantID, tt.wantUUID, tt.wantType, tt.wantEvent,
-					instance.applicationID, instance.allocationUUID, instance.terminationType, instance.event)
+					tt.wantID, tt.wantAllocationID, tt.wantType, tt.wantEvent,
+					instance.applicationID, instance.allocationID, instance.terminationType, instance.event)
 			}
 		})
 	}
@@ -694,16 +694,16 @@ func TestNewReleaseAppAllocationEvent(t *testing.T) {
 
 func TestReleaseAppAllocationEventGetEvent(t *testing.T) {
 	tests := []struct {
-		name                  string
-		appID, allocationUUID string
-		terminationType       si.TerminationType
-		wantEvent             ApplicationEventType
+		name                string
+		appID, allocationID string
+		terminationType     si.TerminationType
+		wantEvent           ApplicationEventType
 	}{
-		{TestEventName, "testAppId001", "testUUID001", si.TerminationType_TIMEOUT, ReleaseAppAllocation},
+		{TestEventName, "testAppId001", "testAllocationID001", si.TerminationType_TIMEOUT, ReleaseAppAllocation},
 	}
 
 	for _, tt := range tests {
-		instance := NewReleaseAppAllocationEvent(tt.appID, tt.terminationType, tt.allocationUUID)
+		instance := NewReleaseAppAllocationEvent(tt.appID, tt.terminationType, tt.allocationID)
 		event := instance.GetEvent()
 		t.Run(tt.name, func(t *testing.T) {
 			if event != tt.wantEvent.String() {
@@ -715,18 +715,18 @@ func TestReleaseAppAllocationEventGetEvent(t *testing.T) {
 
 func TestReleaseAppAllocationEventGetArgs(t *testing.T) {
 	tests := []struct {
-		name                  string
-		appID, allocationUUID string
-		terminationType       si.TerminationType
-		wantLen               int
-		castOk                []bool
-		wantArg               []string
+		name                string
+		appID, allocationID string
+		terminationType     si.TerminationType
+		wantLen             int
+		castOk              []bool
+		wantArg             []string
 	}{
-		{TestArgsName, "testAppId001", "testUUID001", si.TerminationType_TIMEOUT, 2, []bool{true, true}, []string{"testUUID001", "TIMEOUT"}},
+		{TestArgsName, "testAppId001", "testAllocationID001", si.TerminationType_TIMEOUT, 2, []bool{true, true}, []string{"testAllocationID001", "TIMEOUT"}},
 	}
 
 	for _, tt := range tests {
-		instance := NewReleaseAppAllocationEvent(tt.appID, tt.terminationType, tt.allocationUUID)
+		instance := NewReleaseAppAllocationEvent(tt.appID, tt.terminationType, tt.allocationID)
 		args := instance.GetArgs()
 		t.Run(tt.name, func(t *testing.T) {
 			if len(args) != tt.wantLen {
@@ -748,16 +748,16 @@ func TestReleaseAppAllocationEventGetArgs(t *testing.T) {
 
 func TestReleaseAppAllocationEventGetApplicationID(t *testing.T) {
 	tests := []struct {
-		name                  string
-		appID, allocationUUID string
-		terminationType       si.TerminationType
-		wantID                string
+		name                string
+		appID, allocationID string
+		terminationType     si.TerminationType
+		wantID              string
 	}{
-		{TestAppIDName, "testAppId001", "testUUID001", si.TerminationType_TIMEOUT, "testAppId001"},
+		{TestAppIDName, "testAppId001", "testAllocationID001", si.TerminationType_TIMEOUT, "testAppId001"},
 	}
 
 	for _, tt := range tests {
-		instance := NewReleaseAppAllocationEvent(tt.appID, tt.terminationType, tt.allocationUUID)
+		instance := NewReleaseAppAllocationEvent(tt.appID, tt.terminationType, tt.allocationID)
 		appID := instance.GetApplicationID()
 		t.Run(tt.name, func(t *testing.T) {
 			if appID != tt.wantID {
