@@ -34,8 +34,6 @@ import (
 	apis "k8s.io/apimachinery/pkg/apis/meta/v1"
 	k8sEvents "k8s.io/client-go/tools/events"
 
-	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/general"
-	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/client"
 	"github.com/apache/yunikorn-k8shim/pkg/common"
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
@@ -114,7 +112,7 @@ func TestRunApplication(t *testing.T) {
 
 func TestFailApplication(t *testing.T) {
 	context := initContextForTest()
-	dispatcher.RegisterEventHandler(dispatcher.EventTypeApp, context.ApplicationEventHandler())
+	dispatcher.RegisterEventHandler("TestAppHandler", dispatcher.EventTypeApp, context.ApplicationEventHandler())
 	dispatcher.Start()
 	defer dispatcher.Stop()
 
@@ -165,7 +163,6 @@ func TestFailApplication(t *testing.T) {
 			Containers: containers,
 		},
 	}
-	appID := "app-test-001"
 	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task1 := NewTask("task01", app, context, pod)
 	task2 := NewTask("task02", app, context, pod)
@@ -191,8 +188,7 @@ func TestFailApplication(t *testing.T) {
 	assert.Equal(t, rt.time, int64(6))
 	// reset time to 0
 	rt.time = 0
-	appID2 := "app-test-002"
-	app2 := NewApplication(appID2, "root.abc", "testuser", testGroups, map[string]string{}, ms)
+	app2 := NewApplication(app2ID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	app2.SetState(ApplicationStates().New)
 	err = app2.handle(NewFailApplicationEvent(app2.applicationID, errMess))
 	if err == nil {
@@ -213,7 +209,7 @@ func TestFailApplication(t *testing.T) {
 
 func TestSetUnallocatedPodsToFailedWhenFailApplication(t *testing.T) {
 	context := initContextForTest()
-	dispatcher.RegisterEventHandler(dispatcher.EventTypeApp, context.ApplicationEventHandler())
+	dispatcher.RegisterEventHandler("TestAppHandler", dispatcher.EventTypeApp, context.ApplicationEventHandler())
 	dispatcher.Start()
 	defer dispatcher.Stop()
 
@@ -287,7 +283,6 @@ func TestSetUnallocatedPodsToFailedWhenFailApplication(t *testing.T) {
 		},
 	})
 	assert.NilError(t, err)
-	appID := "app-test-001"
 	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task1 := NewTask("task01", app, context, pod1)
 	task2 := NewTaskPlaceholder("task02", app, context, pod2)
@@ -322,7 +317,7 @@ func TestSetUnallocatedPodsToFailedWhenFailApplication(t *testing.T) {
 
 func TestSetUnallocatedPodsToFailedWhenRejectApplication(t *testing.T) {
 	context := initContextForTest()
-	dispatcher.RegisterEventHandler(dispatcher.EventTypeApp, context.ApplicationEventHandler())
+	dispatcher.RegisterEventHandler("TestAppHandler", dispatcher.EventTypeApp, context.ApplicationEventHandler())
 	dispatcher.Start()
 	defer dispatcher.Stop()
 
@@ -382,7 +377,6 @@ func TestSetUnallocatedPodsToFailedWhenRejectApplication(t *testing.T) {
 		},
 	})
 	assert.NilError(t, err)
-	appID := "app-test-001"
 	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task1 := NewTask("task01", app, context, pod1)
 	task2 := NewTask("task02", app, context, pod2)
@@ -391,8 +385,8 @@ func TestSetUnallocatedPodsToFailedWhenRejectApplication(t *testing.T) {
 	app.addTask(task1)
 	app.addTask(task2)
 	app.SetState(ApplicationStates().Submitted)
-	context.AddApplication(&interfaces.AddApplicationRequest{
-		Metadata: interfaces.ApplicationMetadata{
+	context.AddApplication(&AddApplicationRequest{
+		Metadata: ApplicationMetadata{
 			ApplicationID: app.applicationID,
 			QueueName:     app.queue,
 			User:          app.user,
@@ -445,14 +439,12 @@ func TestReleaseAppAllocation(t *testing.T) {
 			Containers: containers,
 		},
 	}
-	appID := "app-test-001"
-	UUID := "testUUID001"
 	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task := NewTask("task01", app, context, pod)
 	app.addTask(task)
-	task.allocationUUID = UUID
+	task.allocationID = taskAllocationID
 	// app must be running states
-	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, UUID))
+	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
 	if err == nil {
 		// this should give an error
 		t.Error("expecting error got 'nil'")
@@ -460,7 +452,7 @@ func TestReleaseAppAllocation(t *testing.T) {
 	// set app states to running, let event can be trigger
 	app.SetState(ApplicationStates().Running)
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
-	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, UUID))
+	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
 	assert.NilError(t, err)
 	// after handle release event the states of app must be running
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
@@ -517,6 +509,9 @@ func (ms *mockSchedulerAPI) UpdateConfiguration(request *si.UpdateConfigurationR
 	return nil
 }
 
+func (ms *mockSchedulerAPI) Stop() {
+}
+
 func assertAppState(t *testing.T, app *Application, expectedState string, duration time.Duration) {
 	deadline := time.Now().Add(duration)
 	for {
@@ -532,12 +527,12 @@ func assertAppState(t *testing.T, app *Application, expectedState string, durati
 
 func TestGetNonTerminatedTaskAlias(t *testing.T) {
 	context := initContextForTest()
-	appID := "app00001"
 	app := NewApplication(appID, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
-	context.addApplication(app)
+	context.addApplicationToContext(app)
 	// app doesn't have any task
 	res := app.getNonTerminatedTaskAlias()
 	assert.Equal(t, len(res), 0)
+	assert.Equal(t, app.AreAllTasksTerminated(), true)
 
 	pod1 := &v1.Pod{
 		TypeMeta: apis.TypeMeta{
@@ -572,6 +567,7 @@ func TestGetNonTerminatedTaskAlias(t *testing.T) {
 	// res should return both task's alias
 	res = app.getNonTerminatedTaskAlias()
 	assert.Equal(t, len(res), 2)
+	assert.Equal(t, app.AreAllTasksTerminated(), false)
 	assert.Assert(t, is.Contains(res, "/test-00001"))
 	assert.Assert(t, is.Contains(res, "/test-00002"))
 
@@ -582,12 +578,14 @@ func TestGetNonTerminatedTaskAlias(t *testing.T) {
 	// res should retuen empty
 	res = app.getNonTerminatedTaskAlias()
 	assert.Equal(t, len(res), 0)
+	assert.Equal(t, app.AreAllTasksTerminated(), true)
 
 	// set two tasks to one is terminated, another is non-terminated
 	task1.sm.SetState(TaskStates().Rejected)
 	task2.sm.SetState(TaskStates().Allocated)
 	// check the task, should only return task2's alias
 	res = app.getNonTerminatedTaskAlias()
+	assert.Equal(t, app.AreAllTasksTerminated(), false)
 	assert.Equal(t, len(res), 1)
 	assert.Equal(t, res[0], "/test-00002")
 }
@@ -597,7 +595,7 @@ func TestSetTaskGroupsAndSchedulingPolicy(t *testing.T) {
 	assert.Equal(t, len(app.getTaskGroups()), 0)
 
 	duration := int64(3000)
-	app.setTaskGroups([]interfaces.TaskGroup{
+	app.setTaskGroups([]TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -687,7 +685,7 @@ func (t *threadSafePodsMap) count() int {
 
 func TestTryReserve(t *testing.T) {
 	context := initContextForTest()
-	dispatcher.RegisterEventHandler(dispatcher.EventTypeApp, context.ApplicationEventHandler())
+	dispatcher.RegisterEventHandler("TestAppHandler", dispatcher.EventTypeApp, context.ApplicationEventHandler())
 	dispatcher.Start()
 	defer dispatcher.Stop()
 
@@ -705,10 +703,10 @@ func TestTryReserve(t *testing.T) {
 	// create a new app
 	app := NewApplication("app00001", "root.abc", "test-user",
 		testGroups, map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
-	context.addApplication(app)
+	context.addApplicationToContext(app)
 
 	// set taskGroups
-	app.setTaskGroups([]interfaces.TaskGroup{
+	app.setTaskGroups([]TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -753,7 +751,7 @@ func TestTryReserve(t *testing.T) {
 
 func TestTryReservePostRestart(t *testing.T) {
 	context := initContextForTest()
-	dispatcher.RegisterEventHandler(dispatcher.EventTypeApp, context.ApplicationEventHandler())
+	dispatcher.RegisterEventHandler("TestAppHandler", dispatcher.EventTypeApp, context.ApplicationEventHandler())
 	dispatcher.Start()
 	defer dispatcher.Stop()
 
@@ -771,10 +769,10 @@ func TestTryReservePostRestart(t *testing.T) {
 	// create a new app
 	app := NewApplication("app00001", "root.abc", "test-user",
 		testGroups, map[string]string{}, mockedAPIProvider.GetAPIs().SchedulerAPI)
-	context.addApplication(app)
+	context.addApplicationToContext(app)
 
 	// set taskGroups
-	app.setTaskGroups([]interfaces.TaskGroup{
+	app.setTaskGroups([]TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -825,7 +823,7 @@ func TestTryReservePostRestart(t *testing.T) {
 			Containers: containers,
 		},
 	})
-	task0.allocationUUID = string(task0.pod.UID)
+	task0.allocationID = string(task0.pod.UID)
 	task0.nodeName = "fake-host"
 	task0.sm.SetState(TaskStates().Allocated)
 
@@ -877,8 +875,8 @@ func TestTryReservePostRestart(t *testing.T) {
 	assert.Equal(t, createdPods.count(), 0)
 }
 
-func TestTriggerAppRecovery(t *testing.T) {
-	// Trigger app recovery should be successful if the app is in New state
+func TestTriggerAppSubmission(t *testing.T) {
+	// Trigger app submission should be successful if the app is in New state
 	mockScheduler := newMockSchedulerAPI()
 	var savedAppRequest *si.ApplicationRequest
 	mockScheduler.UpdateApplicationFn = func(request *si.ApplicationRequest) error {
@@ -899,9 +897,9 @@ func TestTriggerAppRecovery(t *testing.T) {
 	}
 	app.schedulingStyle = "soft"
 
-	err := app.TriggerAppRecovery()
+	err := app.TriggerAppSubmission()
 	assert.NilError(t, err)
-	assert.Equal(t, app.GetApplicationState(), ApplicationStates().Recovering)
+	assert.Equal(t, app.GetApplicationState(), ApplicationStates().Submitted)
 	assert.Assert(t, savedAppRequest != nil, "update function was not called")
 	assert.Equal(t, 1, len(savedAppRequest.New))
 	appRequest := savedAppRequest.New[0]
@@ -923,8 +921,6 @@ func TestTriggerAppRecovery(t *testing.T) {
 	err = app.handle(NewSubmitApplicationEvent(app.applicationID))
 	assert.NilError(t, err)
 	assertAppState(t, app, ApplicationStates().Submitted, 3*time.Second)
-	err = app.TriggerAppRecovery()
-	assert.ErrorContains(t, err, "event RecoverApplication inappropriate in current state Submitted")
 }
 
 func TestSkipReservationStage(t *testing.T) {
@@ -943,7 +939,7 @@ func TestSkipReservationStage(t *testing.T) {
 	task2.sm.SetState(TaskStates().Allocated)
 	app.addTask(task1)
 	app.addTask(task2)
-	app.setTaskGroups([]interfaces.TaskGroup{
+	app.setTaskGroups([]TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -965,7 +961,7 @@ func TestSkipReservationStage(t *testing.T) {
 	task2.sm.SetState(TaskStates().New)
 	app.addTask(task1)
 	app.addTask(task2)
-	app.setTaskGroups([]interfaces.TaskGroup{
+	app.setTaskGroups([]TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 10,
@@ -1003,14 +999,12 @@ func TestReleaseAppAllocationInFailingState(t *testing.T) {
 			Containers: containers,
 		},
 	}
-	appID := "app-test-001"
-	UUID := "testUUID001"
 	app := NewApplication(appID, "root.abc", "testuser", testGroups, map[string]string{}, ms)
 	task := NewTask("task01", app, context, pod)
 	app.addTask(task)
-	task.allocationUUID = UUID
+	task.allocationID = taskAllocationID
 	// app must be running states
-	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, UUID))
+	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
 	if err == nil {
 		// this should give an error
 		t.Error("expecting error got 'nil'")
@@ -1018,12 +1012,12 @@ func TestReleaseAppAllocationInFailingState(t *testing.T) {
 	// set app states to running, let event can be trigger
 	app.SetState(ApplicationStates().Running)
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
-	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, UUID))
+	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
 	assert.NilError(t, err)
 	// after handle release event the states of app must be running
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
 	app.SetState(ApplicationStates().Failing)
-	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, UUID))
+	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, taskAllocationID))
 	assert.NilError(t, err)
 	// after handle release event the states of app must be failing
 	assertAppState(t, app, ApplicationStates().Failing, 3*time.Second)
@@ -1037,7 +1031,7 @@ func TestReleaseAppAllocationInFailingState(t *testing.T) {
 
 func TestResumingStateTransitions(t *testing.T) {
 	context := initContextForTest()
-	dispatcher.RegisterEventHandler(dispatcher.EventTypeApp, context.ApplicationEventHandler())
+	dispatcher.RegisterEventHandler("TestAppHandler", dispatcher.EventTypeApp, context.ApplicationEventHandler())
 	dispatcher.Start()
 	defer dispatcher.Stop()
 
@@ -1063,9 +1057,8 @@ func TestResumingStateTransitions(t *testing.T) {
 	// Add tasks
 	app.addTask(task1)
 	app.addTask(task2)
-	UUID := "testUUID001"
-	task1.allocationUUID = UUID
-	context.addApplication(app)
+	task1.allocationID = taskAllocationID
+	context.addApplicationToContext(app)
 
 	// Set app state to "reserving"
 	app.SetState(ApplicationStates().Reserving)
@@ -1126,10 +1119,6 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 		t.Fatal("the EventRecorder is expected to be of type FakeRecorder")
 	}
 
-	amprotocol := NewMockedAMProtocol()
-	podEvent := general.NewPodEventHandler(amprotocol, false)
-
-	am := general.NewManager(client.NewMockedAPIProvider(false), podEvent)
 	pod1 := v1.Pod{
 		TypeMeta: apis.TypeMeta{
 			Kind:       "Pod",
@@ -1150,8 +1139,8 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 		},
 	}
 
-	// add a pending pod through the AM service
-	am.AddPod(&pod1)
+	// add a pending pod
+	context.AddPod(&pod1)
 
 	pod := &v1.Pod{
 		TypeMeta: apis.TypeMeta{
@@ -1172,24 +1161,18 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 			Phase: v1.PodPending,
 		},
 	}
-	managedApp := amprotocol.GetApplication("app00001")
-	assert.Assert(t, managedApp != nil)
-	app, valid := managedApp.(*Application)
-	if !valid {
-		t.Fatal("application is expected to be of type Application")
-	}
-	assert.Equal(t, valid, true)
+	app := context.GetApplication("app00001")
+	assert.Assert(t, app != nil)
 	assert.Equal(t, app.GetApplicationID(), "app00001")
 	assert.Equal(t, app.GetApplicationState(), ApplicationStates().New)
 	assert.Equal(t, app.GetQueue(), "root.a")
 	assert.Equal(t, len(app.GetNewTasks()), 1)
 
 	appID := "app00001"
-	UUID := "UID-POD-00002"
+	allocationID := "UID-POD-00002"
 
-	context.addApplication(app)
-	task1 := context.AddTask(&interfaces.AddTaskRequest{
-		Metadata: interfaces.TaskMetadata{
+	task1 := context.AddTask(&AddTaskRequest{
+		Metadata: TaskMetadata{
 			ApplicationID: "app00001",
 			TaskID:        "task02",
 			Pod:           pod,
@@ -1202,21 +1185,12 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 	_, taskErr := app.GetTask("task02")
 	assert.NilError(t, taskErr, "Task should exist")
 
-	task2, task2Err := task1.(*Task)
-	if !task2Err {
-		// this should give an error
-		t.Error("task1 is expected to be of type Task")
-	}
-	task2.allocationUUID = UUID
-
-	// app must be running states
-	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, UUID))
-	assert.Error(t, err, "event ReleaseAppAllocation inappropriate in current state New")
+	task1.allocationID = allocationID
 
 	// set app states to running, let event can be trigger
 	app.SetState(ApplicationStates().Running)
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
-	err = app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, UUID))
+	err := app.handle(NewReleaseAppAllocationEvent(appID, si.TerminationType_TIMEOUT, allocationID))
 	assert.NilError(t, err)
 	// after handle release event the states of app must be running
 	assertAppState(t, app, ApplicationStates().Running, 3*time.Second)
@@ -1241,12 +1215,12 @@ func TestPlaceholderTimeoutEvents(t *testing.T) {
 
 func TestApplication_onReservationStateChange(t *testing.T) {
 	context := initContextForTest()
-	dispatcher.RegisterEventHandler(dispatcher.EventTypeApp, context.ApplicationEventHandler())
+	dispatcher.RegisterEventHandler("TestAppHandler", dispatcher.EventTypeApp, context.ApplicationEventHandler())
 	dispatcher.Start()
 	defer dispatcher.Stop()
 
 	app := NewApplication(appID, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
-	context.addApplication(app)
+	context.addApplicationToContext(app)
 
 	app.sm.SetState("Accepted")
 	app.onReservationStateChange()
@@ -1254,7 +1228,7 @@ func TestApplication_onReservationStateChange(t *testing.T) {
 	assertAppState(t, app, ApplicationStates().Running, 1*time.Second)
 
 	// set taskGroups
-	app.setTaskGroups([]interfaces.TaskGroup{
+	app.setTaskGroups([]TaskGroup{
 		{
 			Name:      "test-group-1",
 			MinMember: 1,
@@ -1314,7 +1288,30 @@ func TestApplication_onReservationStateChange(t *testing.T) {
 	assertAppState(t, app, ApplicationStates().Running, 1*time.Second)
 }
 
-func (ctx *Context) addApplication(app *Application) {
+func TestTaskRemoval(t *testing.T) {
+	app := NewApplication(appID, "root.a", "testuser", testGroups, map[string]string{}, newMockSchedulerAPI())
+	context := initContextForTest()
+
+	// "Reserving" state
+	app.SetState(ApplicationStates().Reserving)
+	task := NewTask("task0001", app, context, &v1.Pod{})
+	phTask := NewTaskPlaceholder("ph-task0001", app, context, &v1.Pod{})
+	app.addTask(task)
+	task.sm.SetState(TaskStates().Completed)
+	phTask.sm.SetState(TaskStates().Completed)
+	app.Schedule()
+	assert.Equal(t, 0, len(app.getTasks(TaskStates().Completed)))
+
+	// "Running" state
+	app.SetState(ApplicationStates().Running)
+	task = NewTask("task0002", app, context, &v1.Pod{})
+	app.addTask(task)
+	task.sm.SetState(TaskStates().Completed)
+	app.Schedule()
+	assert.Equal(t, 0, len(app.getTasks(TaskStates().Completed)))
+}
+
+func (ctx *Context) addApplicationToContext(app *Application) {
 	ctx.lock.Lock()
 	defer ctx.lock.Unlock()
 	ctx.applications[app.applicationID] = app
