@@ -123,20 +123,20 @@ func (st SubmitTaskEvent) GetApplicationID() string {
 // Allocate Event
 // ------------------------
 type AllocatedTaskEvent struct {
-	applicationID  string
-	taskID         string
-	event          TaskEventType
-	nodeID         string
-	allocationUUID string
+	applicationID string
+	taskID        string
+	event         TaskEventType
+	nodeID        string
+	allocationID  string
 }
 
-func NewAllocateTaskEvent(appID string, taskID string, allocUUID string, nid string) AllocatedTaskEvent {
+func NewAllocateTaskEvent(appID string, taskID string, allocationID string, nid string) AllocatedTaskEvent {
 	return AllocatedTaskEvent{
-		applicationID:  appID,
-		taskID:         taskID,
-		event:          TaskAllocated,
-		allocationUUID: allocUUID,
-		nodeID:         nid,
+		applicationID: appID,
+		taskID:        taskID,
+		event:         TaskAllocated,
+		allocationID:  allocationID,
+		nodeID:        nid,
 	}
 }
 
@@ -146,7 +146,7 @@ func (ae AllocatedTaskEvent) GetEvent() string {
 
 func (ae AllocatedTaskEvent) GetArgs() []interface{} {
 	args := make([]interface{}, 2)
-	args[0] = ae.allocationUUID
+	args[0] = ae.allocationID
 	args[1] = ae.nodeID
 	return args
 }
@@ -419,15 +419,15 @@ func newTaskState() *fsm.FSM {
 			},
 			beforeHook(TaskAllocated): func(_ context.Context, event *fsm.Event) {
 				task := event.Args[0].(*Task) //nolint:errcheck
-				// All allocation events must include the allocUUID and nodeID passed from the core
+				// All allocation events must include the allocationID and nodeID passed from the core
 				eventArgs := make([]string, 2)
 				if err := events.GetEventArgsAsStrings(eventArgs, event.Args[1].([]interface{})); err != nil {
 					log.Log(log.ShimFSM).Error("failed to parse event arg", zap.Error(err))
 					return
 				}
-				allocUUID := eventArgs[0]
+				allocationID := eventArgs[0]
 				nodeID := eventArgs[1]
-				task.beforeTaskAllocated(event.Src, allocUUID, nodeID)
+				task.beforeTaskAllocated(event.Src, allocationID, nodeID)
 			},
 			beforeHook(CompleteTask): func(_ context.Context, event *fsm.Event) {
 				task := event.Args[0].(*Task) //nolint:errcheck
