@@ -23,10 +23,9 @@ import (
 	"time"
 
 	"go.uber.org/zap"
-	utilfeature "k8s.io/apiserver/pkg/util/feature"
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/tools/cache"
-	"k8s.io/kubernetes/pkg/features"
+	"k8s.io/klog/v2"
 	"k8s.io/kubernetes/pkg/scheduler/framework/plugins/volumebinding"
 
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
@@ -95,16 +94,14 @@ func NewAPIFactory(scheduler api.SchedulerAPI, informerFactory informers.SharedI
 	namespaceInformer := informerFactory.Core().V1().Namespaces()
 	priorityClassInformer := informerFactory.Scheduling().V1().PriorityClasses()
 
-	var capacityCheck volumebinding.CapacityCheck
-	if utilfeature.DefaultFeatureGate.Enabled(features.CSIStorageCapacity) {
-		capacityCheck = volumebinding.CapacityCheck{
-			CSIDriverInformer:          informerFactory.Storage().V1().CSIDrivers(),
-			CSIStorageCapacityInformer: informerFactory.Storage().V1().CSIStorageCapacities(),
-		}
+	var capacityCheck = volumebinding.CapacityCheck{
+		CSIDriverInformer:          informerFactory.Storage().V1().CSIDrivers(),
+		CSIStorageCapacityInformer: informerFactory.Storage().V1().CSIStorageCapacities(),
 	}
 
 	// create a volume binder (needs the informers)
 	volumeBinder := volumebinding.NewVolumeBinder(
+		klog.NewKlogr(),
 		kubeClient.GetClientSet(),
 		podInformer,
 		nodeInformer,
