@@ -19,7 +19,6 @@
 package main
 
 import (
-	"fmt"
 	"os"
 	"os/signal"
 	"syscall"
@@ -32,52 +31,34 @@ import (
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
 	"github.com/apache/yunikorn-k8shim/pkg/log"
 	"github.com/apache/yunikorn-k8shim/pkg/shim"
-	"github.com/apache/yunikorn-scheduler-interface/lib/go/api"
-)
-
-var (
-	version   string
-	date      string
-	goVersion string
-	arch      string
-	coreSHA   string
-	siSHA     string
-	shimSHA   string
 )
 
 func main() {
-	conf.BuildVersion = version
-	conf.BuildDate = date
-	conf.IsPluginVersion = false
-	conf.GoVersion = goVersion
-	conf.Arch = arch
-	conf.CoreSHA = coreSHA
-	conf.SiSHA = siSHA
-	conf.ShimSHA = shimSHA
+	log.Log(log.Shim).Info(conf.GetBuildInfoString())
 
-	log.Logger().Info(fmt.Sprintf("Build info: version=%s date=%s isPluginVersion=%t goVersion=%s arch=%s coreSHA=%s siSHA=%s shimSHA=%s", version, date, false, goVersion, arch, coreSHA, siSHA, shimSHA))
-
-	configMaps, err := client.LoadBootstrapConfigMaps(conf.GetSchedulerNamespace())
+	configMaps, err := client.LoadBootstrapConfigMaps()
 	if err != nil {
-		log.Logger().Fatal("Unable to bootstrap configuration", zap.Error(err))
+		log.Log(log.Shim).Fatal("Unable to bootstrap configuration", zap.Error(err))
 	}
 
 	err = conf.UpdateConfigMaps(configMaps, true)
 	if err != nil {
-		log.Logger().Fatal("Unable to load initial configmaps", zap.Error(err))
+		log.Log(log.Shim).Fatal("Unable to load initial configmaps", zap.Error(err))
 	}
 
-	log.Logger().Info("Starting scheduler", zap.String("name", constants.SchedulerName))
-	serviceContext := entrypoint.StartAllServicesWithLogger(log.Logger(), log.GetZapConfigs())
+	log.Log(log.Shim).Info("Starting scheduler", zap.String("name", constants.SchedulerName))
+	serviceContext := entrypoint.StartAllServicesWithLogger(log.RootLogger(), log.GetZapConfigs())
 
-	if sa, ok := serviceContext.RMProxy.(api.SchedulerAPI); ok {
-		ss := shim.NewShimScheduler(sa, conf.GetSchedulerConf(), configMaps)
-		ss.Run()
+	if serviceContext.RMProxy != nil {
+		ss := shim.NewShimScheduler(serviceContext.RMProxy, conf.GetSchedulerConf(), configMaps)
+		if err := ss.Run(); err != nil {
+			log.Log(log.Shim).Fatal("Unable to start scheduler", zap.Error(err))
+		}
 
 		signalChan := make(chan os.Signal, 1)
 		signal.Notify(signalChan, syscall.SIGINT, syscall.SIGTERM)
 		for range signalChan {
-			log.Logger().Info("Shutdown signal received, exiting...")
+			log.Log(log.Shim).Info("Shutdown signal received, exiting...")
 			ss.Stop()
 			os.Exit(0)
 		}
