@@ -32,7 +32,6 @@ import (
 	schedulingv1 "k8s.io/api/scheduling/v1"
 	podv1 "k8s.io/kubernetes/pkg/api/v1/pod"
 
-	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/common"
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/pkg/conf"
@@ -42,6 +41,7 @@ import (
 )
 
 const userInfoKey = siCommon.DomainYuniKorn + "user.info"
+const uniqueAutogenSuffix = "-uniqueautogen"
 
 var pluginMode bool
 
@@ -77,8 +77,9 @@ func Convert2PriorityClass(obj interface{}) *schedulingv1.PriorityClass {
 	return nil
 }
 
-func NeedRecovery(pod *v1.Pod) bool {
-	// pod requires recovery needs to satisfy both conditions
+// PodAlreadyBound returns true if a newly initializing Pod is already assigned to a Node
+func PodAlreadyBound(pod *v1.Pod) bool {
+	// pod already bound needs to satisfy conditions:
 	// 1. Pod is scheduled by us
 	// 2. pod is already assigned to a node
 	// 3. pod is not in terminated state
@@ -118,7 +119,12 @@ func GetQueueNameFromPod(pod *v1.Pod) string {
 func GenerateApplicationID(namespace string, generateUniqueAppIds bool, podUID string) string {
 	var generatedID string
 	if generateUniqueAppIds {
-		generatedID = fmt.Sprintf("%.26s-%s", namespace, podUID)
+		if podUID == "" {
+			// we must be running in the admission controller and don't yet have a pod UID assigned
+			generatedID = fmt.Sprintf("%.26s%s", namespace, uniqueAutogenSuffix)
+		} else {
+			generatedID = fmt.Sprintf("%.26s-%s", namespace, podUID)
+		}
 	} else {
 		generatedID = fmt.Sprintf("%s-%s-%s", constants.AutoGenAppPrefix, namespace, constants.AutoGenAppSuffix)
 	}
@@ -149,23 +155,32 @@ func GetApplicationIDFromPod(pod *v1.Pod) string {
 	}
 
 	// Application ID can be defined in annotation
-	if value := GetPodAnnotationValue(pod, constants.AnnotationApplicationID); value != "" {
-		return value
+	appID := GetPodAnnotationValue(pod, constants.AnnotationApplicationID)
+	if appID == "" {
+		// Application ID can be defined in label
+		appID = GetPodLabelValue(pod, constants.LabelApplicationID)
 	}
-	// Application ID can be defined in label
-	if value := GetPodLabelValue(pod, constants.LabelApplicationID); value != "" {
-		return value
-	}
-	// Spark can also define application ID
-	if value := GetPodLabelValue(pod, constants.SparkLabelAppID); value != "" {
-		return value
+	if appID == "" {
+		// Spark can also define application ID
+		appID = GetPodLabelValue(pod, constants.SparkLabelAppID)
 	}
 
 	// If plugin mode, interpret missing Application ID as a non-YuniKorn pod
-	if pluginMode {
+	if pluginMode && appID == "" {
 		return ""
 	}
 
+	// does appID end with '-uniqueautogen'?
+	if strings.HasSuffix(appID, uniqueAutogenSuffix) {
+		// replace suffix with pod UID
+		appID = fmt.Sprintf("%s-%s", strings.TrimSuffix(appID, uniqueAutogenSuffix), string(pod.UID))
+	}
+
+	// if app ID is not empty, return it
+	if appID != "" {
+		return appID
+	}
+
 	// Standard deployment mode, so we need a valid Application ID to proceed. Generate one now.
 	return GenerateApplicationID(pod.Namespace, conf.GetSchedulerConf().GenerateUniqueAppIds, string(pod.UID))
 }
@@ -180,16 +195,18 @@ func PodUnderCondition(pod *v1.Pod, condition *v1.PodCondition) bool {
 // get namespace guaranteed resource from namespace annotation
 func GetNamespaceGuaranteedFromAnnotation(namespaceObj *v1.Namespace) *si.Resource {
 	// retrieve guaranteed resource info from annotations
-	namespaceGuaranteed := GetNameSpaceAnnotationValue(namespaceObj, constants.NamespaceGuaranteed)
-	var namespaceGuaranteedMap map[string]string
-	err := json.Unmarshal([]byte(namespaceGuaranteed), &namespaceGuaranteedMap)
-	if err != nil {
-		log.Log(log.ShimUtils).Warn("Unable to process namespace.guaranteed annotation",
-			zap.String("namespace", namespaceObj.Name),
-			zap.String("namespace.guaranteed is", namespaceGuaranteed))
-		return nil
+	if guaranteed := GetNameSpaceAnnotationValue(namespaceObj, constants.NamespaceGuaranteed); guaranteed != "" {
+		var namespaceGuaranteedMap map[string]string
+		err := json.Unmarshal([]byte(guaranteed), &namespaceGuaranteedMap)
+		if err != nil {
+			log.Log(log.ShimUtils).Warn("Unable to process namespace.guaranteed annotation",
+				zap.String("namespace", namespaceObj.Name),
+				zap.String("namespace.guaranteed is", guaranteed))
+			return nil
+		}
+		return common.GetResource(namespaceGuaranteedMap)
 	}
-	return common.GetResource(namespaceGuaranteedMap)
+	return nil
 }
 
 func GetNamespaceQuotaFromAnnotation(namespaceObj *v1.Namespace) *si.Resource {
@@ -363,36 +380,3 @@ func GetPlaceholderFlagFromPodSpec(pod *v1.Pod) bool {
 	}
 	return false
 }
-
-func GetTaskGroupsFromAnnotation(pod *v1.Pod) ([]interfaces.TaskGroup, error) {
-	taskGroupInfo := GetPodAnnotationValue(pod, constants.AnnotationTaskGroups)
-	if taskGroupInfo == "" {
-		return nil, nil
-	}
-
-	taskGroups := []interfaces.TaskGroup{}
-	err := json.Unmarshal([]byte(taskGroupInfo), &taskGroups)
-	if err != nil {
-		return nil, err
-	}
-	// json.Unmarchal won't return error if name or MinMember is empty, but will return error if MinResource is empty or error format.
-	for _, taskGroup := range taskGroups {
-		if taskGroup.Name == "" {
-			return nil, fmt.Errorf("can't get taskGroup Name from pod annotation, %s",
-				taskGroupInfo)
-		}
-		if taskGroup.MinResource == nil {
-			return nil, fmt.Errorf("can't get taskGroup MinResource from pod annotation, %s",
-				taskGroupInfo)
-		}
-		if taskGroup.MinMember == int32(0) {
-			return nil, fmt.Errorf("can't get taskGroup MinMember from pod annotation, %s",
-				taskGroupInfo)
-		}
-		if taskGroup.MinMember < int32(0) {
-			return nil, fmt.Errorf("minMember cannot be negative, %s",
-				taskGroupInfo)
-		}
-	}
-	return taskGroups, nil
-}
