@@ -21,7 +21,6 @@ package utils
 import (
 	"bytes"
 	"compress/gzip"
-	"encoding/base64"
 	"fmt"
 	"strings"
 	"testing"
@@ -30,7 +29,6 @@ import (
 	"gotest.tools/v3/assert"
 	v1 "k8s.io/api/core/v1"
 	schedulingv1 "k8s.io/api/scheduling/v1"
-	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 
 	"github.com/apache/yunikorn-core/pkg/common/configs"
@@ -284,6 +282,21 @@ func TestGetNamespaceGuaranteedFromAnnotation(t *testing.T) {
 				},
 			},
 		}, nil},
+		{&v1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      "test",
+				Namespace: "test",
+				Annotations: map[string]string{
+					constants.NamespaceGuaranteed: "error",
+				},
+			},
+		}, nil},
+		{&v1.Namespace{
+			ObjectMeta: metav1.ObjectMeta{
+				Name:      "test",
+				Namespace: "test",
+			},
+		}, nil},
 	}
 	for _, tc := range testCases {
 		t.Run(fmt.Sprintf("namespace: %v", tc.namespace), func(t *testing.T) {
@@ -532,6 +545,14 @@ func TestGetApplicationIDFromPod(t *testing.T) {
 			},
 			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
 		}, "testns-podUid", "", true},
+		{"Unique autogen token found with generateUnique", &v1.Pod{
+			ObjectMeta: metav1.ObjectMeta{
+				Namespace: "testns",
+				UID:       "podUid",
+				Labels:    map[string]string{constants.LabelApplicationID: "testns-uniqueautogen"},
+			},
+			Spec: v1.PodSpec{SchedulerName: constants.SchedulerName},
+		}, "testns-podUid", "testns-podUid", true},
 		{"Non-yunikorn schedulerName", &v1.Pod{
 			ObjectMeta: metav1.ObjectMeta{
 				Labels: map[string]string{constants.LabelApplicationID: appIDInLabel},
@@ -630,42 +651,79 @@ func TestGenerateApplicationID(t *testing.T) {
 
 	assert.Equal(t, "longlonglonglonglonglonglo-pod-uid",
 		GenerateApplicationID(strings.Repeat("long", 100), true, "pod-uid"))
+
+	assert.Equal(t, "namespace-uniqueautogen",
+		GenerateApplicationID("namespace", true, ""))
 }
 
 func TestMergeMaps(t *testing.T) {
-	result := MergeMaps(nil, nil)
-	assert.Assert(t, result == nil)
+	testCases := []struct {
+		name     string
+		input1   map[string]string
+		input2   map[string]string
+		expected map[string]string
+	}{
+		{
+			name:     "Both inputs nil",
+			input1:   nil,
+			input2:   nil,
+			expected: nil,
+		},
+		{
+			name:   "First input nil, Second input not nil",
+			input1: nil,
+			input2: map[string]string{"a": "b"},
+			expected: map[string]string{
+				"a": "b",
+			},
+		},
+		{
+			name:   "First input not nil, Second input nil",
+			input1: map[string]string{"a": "b"},
+			input2: nil,
+			expected: map[string]string{
+				"a": "b",
+			},
+		},
+		{
+			name: "Merge with existing key",
+			input1: map[string]string{
+				"a": "a1",
+			},
+			input2: map[string]string{
+				"a": "a2",
+			},
+			expected: map[string]string{
+				"a": "a2",
+			},
+		},
+		{
+			name: "Merge with additional keys",
+			input1: map[string]string{
+				"a": "a1",
+				"b": "b1",
+				"c": "c1",
+			},
+			input2: map[string]string{
+				"a": "a2",
+				"b": "b2",
+				"d": "d2",
+			},
+			expected: map[string]string{
+				"a": "a2",
+				"b": "b2",
+				"c": "c1",
+				"d": "d2",
+			},
+		},
+	}
 
-	result = MergeMaps(nil, map[string]string{"a": "b"})
-	assert.Assert(t, result != nil)
-	assert.Equal(t, len(result), 1)
-	assert.Equal(t, result["a"], "b")
-
-	result = MergeMaps(map[string]string{"a": "b"}, nil)
-	assert.Assert(t, result != nil)
-	assert.Equal(t, len(result), 1)
-	assert.Equal(t, result["a"], "b")
-
-	result = MergeMaps(map[string]string{"a": "a1"}, map[string]string{"a": "a2"})
-	assert.Assert(t, result != nil)
-	assert.Equal(t, len(result), 1)
-	assert.Equal(t, result["a"], "a2")
-
-	result = MergeMaps(map[string]string{
-		"a": "a1",
-		"b": "b1",
-		"c": "c1",
-	}, map[string]string{
-		"a": "a2",
-		"b": "b2",
-		"d": "d2",
-	})
-	assert.Assert(t, result != nil)
-	assert.Equal(t, len(result), 4)
-	assert.Equal(t, result["a"], "a2")
-	assert.Equal(t, result["b"], "b2")
-	assert.Equal(t, result["c"], "c1")
-	assert.Equal(t, result["d"], "d2")
+	for _, tc := range testCases {
+		t.Run(tc.name, func(t *testing.T) {
+			result := MergeMaps(tc.input1, tc.input2)
+			assert.DeepEqual(t, result, tc.expected)
+		})
+	}
 }
 
 func TestGetUserFromPodLabel(t *testing.T) {
@@ -811,12 +869,12 @@ func TestGetQueueNameFromPod(t *testing.T) {
 	}
 }
 
-func TestNeedRecovery(t *testing.T) {
+func TestPodAlreadyBound(t *testing.T) {
 	const fakeNodeID = "fake-node"
 	testCases := []struct {
-		description          string
-		pod                  *v1.Pod
-		expectedRecoveryFlag bool
+		description       string
+		pod               *v1.Pod
+		expectedBoundFlag bool
 	}{
 		{"New pod pending for scheduling",
 			&v1.Pod{
@@ -882,8 +940,8 @@ func TestNeedRecovery(t *testing.T) {
 
 	for _, tc := range testCases {
 		t.Run(tc.description, func(t *testing.T) {
-			recovery := NeedRecovery(tc.pod)
-			assert.Equal(t, recovery, tc.expectedRecoveryFlag, tc.description)
+			bound := PodAlreadyBound(tc.pod)
+			assert.Equal(t, bound, tc.expectedBoundFlag, tc.description)
 		})
 	}
 }
@@ -986,190 +1044,6 @@ func TestGetPlaceholderFlagFromPodSpec(t *testing.T) {
 	}
 }
 
-// nolint: funlen
-func TestGetTaskGroupFromAnnotation(t *testing.T) {
-	// correct json
-	testGroup := `
-	[
-		{
-			"name": "test-group-1",
-			"minMember": 10,
-			"minResource": {
-				"cpu": 1,
-				"memory": "2Gi"
-			},
-			"nodeSelector": {
-				"test": "testnode",
-				"locate": "west"
-			},
-			"tolerations": [
-				{
-					"key": "key",
-					"operator": "Equal",
-					"value": "value",
-					"effect": "NoSchedule"
-				}
-			]
-		},
-		{
-			"name": "test-group-2",
-			"minMember": 5,
-			"minResource": {
-				"cpu": 2,
-				"memory": "4Gi"
-			}
-		}
-	]`
-	testGroup2 := `
-	[
-		{
-			"name": "test-group-3",
-			"minMember": 3,
-			"minResource": {
-				"cpu": 2,
-				"memory": "1Gi"
-			}
-		}
-	]`
-	// Error json
-	testGroupErr := `
-	[
-		{
-			"name": "test-group-err-1",
-			"minMember": "ERR",
-			"minResource": {
-				"cpu": "ERR",
-				"memory": "ERR"
-			},
-		}
-	]`
-	// without name
-	testGroupErr2 := `
-	[
-		{
-			"minMember": 3,
-			"minResource": {
-				"cpu": 2,
-				"memory": "1Gi"
-			}
-		}
-	]`
-	// without minMember
-	testGroupErr3 := `
-	[
-		{
-			"name": "test-group-err-3",
-			"minResource": {
-				"cpu": 2,
-				"memory": "1Gi"
-			}
-		}
-	]`
-	// without minResource
-	testGroupErr4 := `
-	[
-		{
-			"name": "test-group-err-4",
-			"minMember": 3
-		}
-	]`
-	// negative minMember without minResource
-	testGroupErr5 := `
-	[
-		{
-			"name": "test-group-err-5",
-			"minMember": -100
-		}
-	]`
-	// negative minMember with minResource
-	testGroupErr6 := `
-	[
-		{
-			"name": "test-group-err-6",
-			"minMember": -100,
-			"minResource": {
-				"cpu": 2,
-				"memory": "1Gi"
-			}
-		}
-	]`
-	// Insert task group info to pod annotation
-	pod := &v1.Pod{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      "test-pod-err",
-			Namespace: "test",
-			UID:       "test-pod-UID-err",
-		},
-		Spec: v1.PodSpec{},
-		Status: v1.PodStatus{
-			Phase: v1.PodPending,
-		},
-	}
-	// Empty case
-	taskGroupEmpty, err := GetTaskGroupsFromAnnotation(pod)
-	assert.Assert(t, taskGroupEmpty == nil)
-	assert.Assert(t, err == nil)
-	// Error case
-	pod.Annotations = map[string]string{constants.AnnotationTaskGroups: testGroupErr}
-	taskGroupErr, err := GetTaskGroupsFromAnnotation(pod)
-	assert.Assert(t, taskGroupErr == nil)
-	assert.Assert(t, err != nil)
-	pod.Annotations = map[string]string{constants.AnnotationTaskGroups: testGroupErr2}
-	taskGroupErr2, err := GetTaskGroupsFromAnnotation(pod)
-	assert.Assert(t, taskGroupErr2 == nil)
-	assert.Assert(t, err != nil)
-	pod.Annotations = map[string]string{constants.AnnotationTaskGroups: testGroupErr3}
-	taskGroupErr3, err := GetTaskGroupsFromAnnotation(pod)
-	assert.Assert(t, taskGroupErr3 == nil)
-	assert.Assert(t, err != nil)
-	pod.Annotations = map[string]string{constants.AnnotationTaskGroups: testGroupErr4}
-	taskGroupErr4, err := GetTaskGroupsFromAnnotation(pod)
-	assert.Assert(t, taskGroupErr4 == nil)
-	assert.Assert(t, err != nil)
-	pod.Annotations = map[string]string{constants.AnnotationTaskGroups: testGroupErr5}
-	taskGroupErr5, err := GetTaskGroupsFromAnnotation(pod)
-	assert.Assert(t, taskGroupErr5 == nil)
-	assert.Assert(t, err != nil)
-	pod.Annotations = map[string]string{constants.AnnotationTaskGroups: testGroupErr6}
-	taskGroupErr6, err := GetTaskGroupsFromAnnotation(pod)
-	assert.Assert(t, taskGroupErr6 == nil)
-	assert.Assert(t, err != nil)
-	// Correct case
-	pod.Annotations = map[string]string{constants.AnnotationTaskGroups: testGroup}
-	taskGroups, err := GetTaskGroupsFromAnnotation(pod)
-	assert.NilError(t, err)
-	// Group value check
-	assert.Equal(t, taskGroups[0].Name, "test-group-1")
-	assert.Equal(t, taskGroups[0].MinMember, int32(10))
-	assert.Equal(t, taskGroups[0].MinResource["cpu"], resource.MustParse("1"))
-	assert.Equal(t, taskGroups[0].MinResource["memory"], resource.MustParse("2Gi"))
-	assert.Equal(t, taskGroups[1].Name, "test-group-2")
-	assert.Equal(t, taskGroups[1].MinMember, int32(5))
-	assert.Equal(t, taskGroups[1].MinResource["cpu"], resource.MustParse("2"))
-	assert.Equal(t, taskGroups[1].MinResource["memory"], resource.MustParse("4Gi"))
-	// NodeSelector check
-	assert.Equal(t, taskGroups[0].NodeSelector["test"], "testnode")
-	assert.Equal(t, taskGroups[0].NodeSelector["locate"], "west")
-	// Toleration check
-	var tolerations []v1.Toleration
-	toleration := v1.Toleration{
-		Key:      "key",
-		Operator: "Equal",
-		Value:    "value",
-		Effect:   "NoSchedule",
-	}
-	tolerations = append(tolerations, toleration)
-	assert.DeepEqual(t, taskGroups[0].Tolerations, tolerations)
-
-	pod.Annotations = map[string]string{constants.AnnotationTaskGroups: testGroup2}
-	taskGroups2, err := GetTaskGroupsFromAnnotation(pod)
-	assert.NilError(t, err)
-	assert.Equal(t, taskGroups2[0].Name, "test-group-3")
-	assert.Equal(t, taskGroups2[0].MinMember, int32(3))
-	assert.Equal(t, taskGroups2[0].MinResource["cpu"], resource.MustParse("2"))
-	assert.Equal(t, taskGroups2[0].MinResource["memory"], resource.MustParse("1Gi"))
-}
-
 func TestGetCoreSchedulerConfigFromConfigMapNil(t *testing.T) {
 	assert.Equal(t, "", GetCoreSchedulerConfigFromConfigMap(nil))
 }
@@ -1195,11 +1069,9 @@ func TestGzipCompressedConfigMap(t *testing.T) {
 	if err := gzWriter.Close(); err != nil {
 		t.Fatal("expected nil, got error")
 	}
-	encodedConfigString := make([]byte, base64.StdEncoding.EncodedLen(len(b.Bytes())))
-	base64.StdEncoding.Encode(encodedConfigString, b.Bytes())
 	confMap := conf.FlattenConfigMaps([]*v1.ConfigMap{
 		{Data: map[string]string{}},
-		{Data: map[string]string{conf.CMSvcClusterID: "new"}, BinaryData: map[string][]byte{"queues.yaml.gz": encodedConfigString}},
+		{Data: map[string]string{conf.CMSvcClusterID: "new"}, BinaryData: map[string][]byte{"queues.yaml.gz": b.Bytes()}},
 	})
 	config := GetCoreSchedulerConfigFromConfigMap(confMap)
 	assert.Equal(t, configs.DefaultSchedulerConfig, config)
