@@ -21,7 +21,6 @@ package conf
 import (
 	"bytes"
 	"compress/gzip"
-	"encoding/base64"
 	"encoding/json"
 	"errors"
 	"flag"
@@ -64,7 +63,6 @@ const (
 	CMSvcVolumeBindTimeout            = PrefixService + "volumeBindTimeout"
 	CMSvcEventChannelCapacity         = PrefixService + "eventChannelCapacity"
 	CMSvcDispatchTimeout              = PrefixService + "dispatchTimeout"
-	CMSvcOperatorPlugins              = PrefixService + "operatorPlugins"
 	CMSvcDisableGangScheduling        = PrefixService + "disableGangScheduling"
 	CMSvcEnableConfigHotRefresh       = PrefixService + "enableConfigHotRefresh"
 	CMSvcPlaceholderImage             = PrefixService + "placeholderImage"
@@ -123,7 +121,6 @@ type SchedulerConf struct {
 	DispatchTimeout          time.Duration `json:"dispatchTimeout"`
 	KubeQPS                  int           `json:"kubeQPS"`
 	KubeBurst                int           `json:"kubeBurst"`
-	OperatorPlugins          string        `json:"operatorPlugins"`
 	EnableConfigHotRefresh   bool          `json:"enableConfigHotRefresh"`
 	DisableGangScheduling    bool          `json:"disableGangScheduling"`
 	UserLabelKey             string        `json:"userLabelKey"`
@@ -151,7 +148,6 @@ func (conf *SchedulerConf) Clone() *SchedulerConf {
 		DispatchTimeout:          conf.DispatchTimeout,
 		KubeQPS:                  conf.KubeQPS,
 		KubeBurst:                conf.KubeBurst,
-		OperatorPlugins:          conf.OperatorPlugins,
 		EnableConfigHotRefresh:   conf.EnableConfigHotRefresh,
 		DisableGangScheduling:    conf.DisableGangScheduling,
 		UserLabelKey:             conf.UserLabelKey,
@@ -212,7 +208,6 @@ func handleNonReloadableConfig(old *SchedulerConf, new *SchedulerConf) {
 	checkNonReloadableDuration(CMSvcDispatchTimeout, &old.DispatchTimeout, &new.DispatchTimeout)
 	checkNonReloadableInt(CMKubeQPS, &old.KubeQPS, &new.KubeQPS)
 	checkNonReloadableInt(CMKubeBurst, &old.KubeBurst, &new.KubeBurst)
-	checkNonReloadableString(CMSvcOperatorPlugins, &old.OperatorPlugins, &new.OperatorPlugins)
 	checkNonReloadableBool(CMSvcDisableGangScheduling, &old.DisableGangScheduling, &new.DisableGangScheduling)
 	checkNonReloadableString(CMSvcPlaceholderImage, &old.PlaceHolderImage, &new.PlaceHolderImage)
 	checkNonReloadableString(CMSvcNodeInstanceTypeNodeLabelKey, &old.InstanceTypeNodeLabelKey, &new.InstanceTypeNodeLabelKey)
@@ -284,23 +279,6 @@ func (conf *SchedulerConf) GetKubeConfigPath() string {
 	return conf.KubeConfig
 }
 
-func (conf *SchedulerConf) IsOperatorPluginEnabled(name string) bool {
-	conf.RLock()
-	defer conf.RUnlock()
-	if conf.OperatorPlugins == "" {
-		return false
-	}
-
-	plugins := strings.Split(conf.OperatorPlugins, ",")
-	for _, p := range plugins {
-		if p == name {
-			return true
-		}
-	}
-
-	return false
-}
-
 func GetSchedulerNamespace() string {
 	if value, ok := os.LookupEnv(EnvNamespace); ok {
 		return value
@@ -340,7 +318,6 @@ func CreateDefaultConfig() *SchedulerConf {
 		DispatchTimeout:          DefaultDispatchTimeout,
 		KubeQPS:                  DefaultKubeQPS,
 		KubeBurst:                DefaultKubeBurst,
-		OperatorPlugins:          DefaultOperatorPlugins,
 		EnableConfigHotRefresh:   DefaultEnableConfigHotRefresh,
 		DisableGangScheduling:    DefaultDisableGangScheduling,
 		UserLabelKey:             constants.DefaultUserLabel,
@@ -367,7 +344,6 @@ func parseConfig(config map[string]string, prev *SchedulerConf) (*SchedulerConf,
 	parser.durationVar(&conf.VolumeBindTimeout, CMSvcVolumeBindTimeout)
 	parser.intVar(&conf.EventChannelCapacity, CMSvcEventChannelCapacity)
 	parser.durationVar(&conf.DispatchTimeout, CMSvcDispatchTimeout)
-	parser.stringVar(&conf.OperatorPlugins, CMSvcOperatorPlugins)
 	parser.boolVar(&conf.DisableGangScheduling, CMSvcDisableGangScheduling)
 	parser.boolVar(&conf.EnableConfigHotRefresh, CMSvcEnableConfigHotRefresh)
 	parser.stringVar(&conf.PlaceHolderImage, CMSvcPlaceholderImage)
@@ -475,18 +451,10 @@ func DumpConfiguration() {
 
 func Decompress(key string, value []byte) (string, string) {
 	var uncompressedData string
-	decodedValue := make([]byte, base64.StdEncoding.DecodedLen(len(value)))
-	n, err := base64.StdEncoding.Decode(decodedValue, value)
-	if err != nil {
-		log.Log(log.ShimConfig).Error("failed to decode schedulerConfig entry", zap.Error(err))
-		return "", ""
-	}
-	decodedValue = decodedValue[:n]
 	splitKey := strings.Split(key, ".")
 	compressionAlgo := splitKey[len(splitKey)-1]
 	if strings.EqualFold(compressionAlgo, constants.GzipSuffix) {
-		reader := bytes.NewReader(decodedValue)
-		gzReader, err := gzip.NewReader(reader)
+		gzReader, err := gzip.NewReader(bytes.NewReader(value))
 		if err != nil {
 			log.Log(log.ShimConfig).Error("failed to decompress decoded schedulerConfig entry", zap.Error(err))
 			return "", ""
