@@ -28,10 +28,10 @@ import (
 	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/client-go/informers"
+	"k8s.io/klog/v2"
 	"k8s.io/kubernetes/pkg/scheduler/framework"
 
 	"github.com/apache/yunikorn-core/pkg/entrypoint"
-	"github.com/apache/yunikorn-k8shim/pkg/appmgmt/interfaces"
 	"github.com/apache/yunikorn-k8shim/pkg/cache"
 	"github.com/apache/yunikorn-k8shim/pkg/client"
 	"github.com/apache/yunikorn-k8shim/pkg/common/events"
@@ -121,12 +121,12 @@ func (sp *YuniKornSchedulerPlugin) PreEnqueue(_ context.Context, pod *v1.Pod) *f
 
 		schedState := task.GetTaskSchedulingState()
 		switch schedState {
-		case interfaces.TaskSchedPending:
+		case cache.TaskSchedPending:
 			return framework.NewStatus(framework.UnschedulableAndUnresolvable, "Pod is pending scheduling")
-		case interfaces.TaskSchedFailed:
+		case cache.TaskSchedFailed:
 			// allow the pod to proceed so that it will be marked unschedulable by PreFilter
 			return nil
-		case interfaces.TaskSchedSkipped:
+		case cache.TaskSchedSkipped:
 			return framework.NewStatus(framework.UnschedulableAndUnresolvable, "Pod doesn't fit within queue")
 		default:
 			return framework.NewStatus(framework.UnschedulableAndUnresolvable, fmt.Sprintf("Pod unschedulable: %s", schedState.String()))
@@ -168,7 +168,7 @@ func (sp *YuniKornSchedulerPlugin) PreFilter(_ context.Context, _ *framework.Cyc
 				zap.String("pod", pod.Name),
 				zap.String("taskID", taskID),
 				zap.String("assignedNode", nodeID))
-			return &framework.PreFilterResult{NodeNames: sets.NewString(nodeID)}, nil
+			return &framework.PreFilterResult{NodeNames: sets.New[string](nodeID)}, nil
 		}
 	}
 
@@ -217,8 +217,26 @@ func (sp *YuniKornSchedulerPlugin) Filter(_ context.Context, _ *framework.CycleS
 	return framework.NewStatus(framework.UnschedulableAndUnresolvable, "Pod is not fit for node")
 }
 
-func (sp *YuniKornSchedulerPlugin) EventsToRegister() []framework.ClusterEvent {
-	return sp.context.EventsToRegister()
+func (sp *YuniKornSchedulerPlugin) EventsToRegister() []framework.ClusterEventWithHint {
+	return sp.context.EventsToRegister(func(_ klog.Logger, pod *v1.Pod, _, _ interface{}) (framework.QueueingHint, error) {
+		// adapt our simpler function to the QueueingHintFn contract
+		return sp.queueingHint(pod)
+	})
+}
+
+// queueingHint is used to perform a lightweight check to determine if any object change may cause a pod to become
+// schedulable when it was not previously. Since YuniKorn maintains its own internal scheduling state, only the pod
+// is needed. This function will only be called on a previously unschedulable pod by this plugin -- therefore this
+// is definitely a YuniKorn pod.
+func (sp *YuniKornSchedulerPlugin) queueingHint(pod *v1.Pod) (framework.QueueingHint, error) {
+	// Use the context's bloom filter to rule out this task if it is not present. Given a large backlog,
+	// this will almost always return false and we can skip re-enqueue.
+	taskID := string(pod.UID)
+	if !sp.context.IsTaskMaybeSchedulable(taskID) {
+		return framework.QueueSkip, nil
+	}
+
+	return framework.Queue, nil
 }
 
 // PostBind is used to mark allocations as completed once scheduling run is finished
@@ -249,7 +267,7 @@ func (sp *YuniKornSchedulerPlugin) PostBind(_ context.Context, _ *framework.Cycl
 }
 
 // NewSchedulerPlugin initializes a new plugin and returns it
-func NewSchedulerPlugin(_ runtime.Object, handle framework.Handle) (framework.Plugin, error) {
+func NewSchedulerPlugin(_ context.Context, _ runtime.Object, handle framework.Handle) (framework.Plugin, error) {
 	log.Log(log.ShimSchedulerPlugin).Info(conf.GetBuildInfoString())
 
 	configMaps, err := client.LoadBootstrapConfigMaps()
@@ -282,7 +300,7 @@ func NewSchedulerPlugin(_ runtime.Object, handle framework.Handle) (framework.Pl
 	return p, nil
 }
 
-func (sp *YuniKornSchedulerPlugin) getTask(appID, taskID string) (app interfaces.ManagedApp, task interfaces.ManagedTask, ok bool) {
+func (sp *YuniKornSchedulerPlugin) getTask(appID, taskID string) (app *cache.Application, task *cache.Task, ok bool) {
 	if app := sp.context.GetApplication(appID); app != nil {
 		if task, err := app.GetTask(taskID); err == nil {
 			return app, task, true
@@ -291,7 +309,7 @@ func (sp *YuniKornSchedulerPlugin) getTask(appID, taskID string) (app interfaces
 	return nil, nil, false
 }
 
-func (sp *YuniKornSchedulerPlugin) failTask(pod *v1.Pod, app interfaces.ManagedApp, task interfaces.ManagedTask) {
+func (sp *YuniKornSchedulerPlugin) failTask(pod *v1.Pod, app *cache.Application, task *cache.Task) {
 	taskID := task.GetTaskID()
 	log.Log(log.ShimSchedulerPlugin).Info("Task failed scheduling, marking as rejected",
 		zap.String("namespace", pod.Namespace),
@@ -299,5 +317,5 @@ func (sp *YuniKornSchedulerPlugin) failTask(pod *v1.Pod, app interfaces.ManagedA
 		zap.String("taskID", taskID))
 	sp.context.RemovePodAllocation(taskID)
 	dispatcher.Dispatch(cache.NewRejectTaskEvent(app.GetApplicationID(), taskID, fmt.Sprintf("task %s rejected by scheduler", taskID)))
-	task.SetTaskSchedulingState(interfaces.TaskSchedFailed)
+	task.SetTaskSchedulingState(cache.TaskSchedFailed)
 }
