@@ -25,7 +25,6 @@ import (
 	v1 "k8s.io/api/core/v1"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/types"
 
 	"github.com/apache/yunikorn-k8shim/pkg/common/constants"
 	"github.com/apache/yunikorn-k8shim/test/e2e/framework/configmanager"
@@ -40,11 +39,18 @@ type SleepPodConfig struct {
 	CPU          int64
 	Mem          int64
 	RequiredNode string
-	Optedout     bool
-	UID          types.UID
+	Optedout     AllowPreemptOpted
 	Labels       map[string]string
 }
 
+type AllowPreemptOpted int
+
+const (
+	NotConfig AllowPreemptOpted = iota
+	Allow
+	Deny
+)
+
 // TestPodConfig template for  sleepPods
 func InitSleepPod(conf SleepPodConfig) (*v1.Pod, error) {
 	// Initialize default values
@@ -90,15 +96,21 @@ func InitSleepPod(conf SleepPodConfig) (*v1.Pod, error) {
 		}
 	}
 
-	optedOut := "true"
-	if !conf.Optedout {
-		optedOut = "false"
+	annotation := &PodAnnotation{
+		Other: map[string]string{},
+	}
+
+	switch conf.Optedout {
+	case NotConfig:
+	case Allow:
+		annotation.Other[constants.AnnotationAllowPreemption] = constants.True
+	case Deny:
+		annotation.Other[constants.AnnotationAllowPreemption] = constants.False
 	}
 
 	labels := map[string]string{
-		"app":                                  "sleep",
-		"applicationId":                        conf.AppID,
-		"yunikorn.apache.org/allow-preemption": optedOut,
+		"app":           "sleep",
+		"applicationId": conf.AppID,
 	}
 	if len(conf.Labels) > 0 {
 		for k, v := range conf.Labels {
@@ -112,6 +124,7 @@ func InitSleepPod(conf SleepPodConfig) (*v1.Pod, error) {
 		RestartPolicy:              v1.RestartPolicyNever,
 		DeletionGracePeriodSeconds: &secs,
 		Command:                    []string{"sleep", strconv.Itoa(conf.Time)},
+		Annotations:                annotation,
 		Labels:                     labels,
 		Resources: &v1.ResourceRequirements{
 			Requests: v1.ResourceList{
@@ -145,6 +158,9 @@ type TestPodConfig struct {
 	RestartPolicy              v1.RestartPolicy
 	Command                    []string
 	InitContainerSleepSecs     int
+	PvcName                    string
+	PvName                     string
+	VolumeName                 string
 }
 
 func InitTestPod(conf TestPodConfig) (*v1.Pod, error) { //nolint:funlen
@@ -214,6 +230,25 @@ func InitTestPod(conf TestPodConfig) (*v1.Pod, error) { //nolint:funlen
 			},
 		}
 	}
+	if conf.PvcName != "" || conf.PvName != "" {
+		if conf.VolumeName == "" {
+			conf.VolumeName = "vol-" + common.RandSeq(5)
+		}
+		if conf.PvcName != "" {
+			pod.Spec.Volumes = []v1.Volume{
+				{
+					Name: conf.VolumeName,
+					VolumeSource: v1.VolumeSource{
+						PersistentVolumeClaim: &v1.PersistentVolumeClaimVolumeSource{
+							ClaimName: conf.PvcName,
+						},
+					},
+				},
+			}
+		} else if conf.PvName != "" {
+			pod.Spec.Volumes = []v1.Volume{}
+		}
+	}
 	return pod, nil
 }
 
