@@ -23,7 +23,7 @@ import {AppInfo} from '@app/models/app-info.model';
 import {ClusterInfo} from '@app/models/cluster-info.model';
 import {HistoryInfo} from '@app/models/history-info.model';
 import {NodeInfo} from '@app/models/node-info.model';
-import {NodeUtilization} from '@app/models/node-utilization.model';
+import {NodeUtilizationsInfo} from '@app/models/node-utilization.model';
 import {Partition} from '@app/models/partition-info.model';
 
 import {QueueInfo, QueuePropertyItem} from '@app/models/queue-info.model';
@@ -125,7 +125,7 @@ export class SchedulerService {
                     alloc['displayName'],
                     alloc['allocationKey'],
                     alloc['allocationTags'],
-                    alloc['uuid'],
+                    alloc['allocationID'],
                     this.formatResource(alloc['resource'] as SchedulerResourceInfo),
                     alloc['priority'],
                     alloc['queueName'],
@@ -205,7 +205,8 @@ export class SchedulerService {
               this.formatResource(node['occupied'] as SchedulerResourceInfo),
               this.formatResource(node['available'] as SchedulerResourceInfo),
               this.formatPercent(node['utilized'] as SchedulerResourceInfo),
-              []
+              [],
+              node['attributes'],
             );
 
             const allocations = node['allocations'];
@@ -231,7 +232,7 @@ export class SchedulerService {
                     alloc['displayName'],
                     alloc['allocationKey'],
                     alloc['allocationTags'],
-                    alloc['uuid'],
+                    alloc['allocationID'],
                     this.formatResource(alloc['resource'] as SchedulerResourceInfo),
                     alloc['priority'],
                     alloc['queueName'],
@@ -254,9 +255,9 @@ export class SchedulerService {
     );
   }
 
-  fetchNodeUtilization(): Observable<NodeUtilization>{
-    const nodeUtilizationUrl = `${this.envConfig.getSchedulerWebAddress()}/ws/v1/scheduler/node-utilization`;
-    return this.httpClient.get(nodeUtilizationUrl).pipe(map((data: any) => data as NodeUtilization));
+  fetchNodeUtilizationsInfo(): Observable<NodeUtilizationsInfo[]>{
+    const nodeUtilizationsUrl = `${this.envConfig.getSchedulerWebAddress()}/ws/v1/scheduler/node-utilizations`;
+    return this.httpClient.get(nodeUtilizationsUrl).pipe(map((data: any) => data as NodeUtilizationsInfo[]));
   }
 
   fecthHealthchecks(): Observable<SchedulerHealthInfo> {
@@ -325,65 +326,58 @@ export class SchedulerService {
   }
 
   private formatResource(resource: SchedulerResourceInfo): string {
-    const formatted = [];
+    const formatted: string[] = [];
+    if (resource) {
+      // Object.keys() didn't guarantee the order of keys, sort it before iterate.
+      Object.keys(resource).sort(CommonUtil.resourcesCompareFn).forEach((key) => {
+        let value = resource[key];
+        let formattedKey = key;
+        let formattedValue : string;
 
-    if (resource && resource.memory !== undefined) {
-      formatted.push(`Memory: ${CommonUtil.formatBytes(resource.memory)}`);
-    } else {
-      formatted.push(`Memory: ${NOT_AVAILABLE}`);
-    }
-
-    if (resource && resource.vcore !== undefined) {
-      formatted.push(`CPU: ${CommonUtil.formatCount(resource.vcore)}`);
-    } else {
-      formatted.push(`CPU: ${NOT_AVAILABLE}`);
-    }
-
-    if (resource){
-      Object.keys(resource).forEach((key) => {
         switch(key){
           case "memory":
-          case "vcore":{
+            formattedKey = "Memory";
+            formattedValue = CommonUtil.formatMemoryBytes(value);
             break;
-          }
-          case "ephemeral-storage":{
-            if (resource[`ephemeral-storage`] == 0) {
-              formatted.push(`ephemeral-storage: ${NOT_AVAILABLE}`);
-            }else{
-              formatted.push(`ephemeral-storage: ${CommonUtil.formatBytes(resource[key])}`);
+          case "vcore":
+            formattedKey = "CPU";
+            formattedValue = CommonUtil.formatCpuCore(value);
+            break;
+          case "ephemeral-storage":
+            formattedValue = CommonUtil.formatEphemeralStorageBytes(value);
+            break;
+          default:
+            if (key.startsWith('hugepages-')) {
+              formattedValue = CommonUtil.formatMemoryBytes(value);
+            } else{
+              formattedValue = CommonUtil.formatOtherResource(value);
             }
             break;
-          }
-          default:{
-            if (resource[key] == 0) {
-              formatted.push(`${key}: ${NOT_AVAILABLE}`);
-            }else{
-              formatted.push(`${key}: ${CommonUtil.formatOtherResource(resource[key])}`);
-            }
-            break;
-          }
-        }
+         }
+        formatted.push(`${formattedKey}: ${formattedValue}`);
       });
     }
-    
+
+    if (formatted.length === 0) {
+      return NOT_AVAILABLE;
+    }
     return formatted.join(', ');
-  }
+  } 
 
   private formatPercent(resource: SchedulerResourceInfo): string {
     const formatted = [];
 
-    if (resource && resource.memory !== undefined) {
-      formatted.push(`Memory: ${CommonUtil.formatPercent(resource.memory)}`);
-    } else {
-      formatted.push(`Memory: ${NOT_AVAILABLE}`);
+    if (resource) {
+      if (resource.memory !== undefined){
+        formatted.push(`Memory: ${CommonUtil.formatPercent(resource.memory)}`);
+      }
+      if (resource.vcore !== undefined){
+        formatted.push(`CPU: ${CommonUtil.formatPercent(resource.vcore)}`);
+      }
     }
-
-    if (resource && resource.vcore !== undefined) {
-      formatted.push(`CPU: ${CommonUtil.formatPercent(resource.vcore)}`);
-    } else {
-      formatted.push(`CPU: ${NOT_AVAILABLE}`);
+    if (formatted.length === 0) {
+      return NOT_AVAILABLE;
     }
-
     return formatted.join(', ');
   }
 
